@@ -1,5 +1,4 @@
-/*  $Id: ckpasswd.c 9587 2013-12-14 20:10:48Z iulius $
-**
+/*
 **  The default username/password authenticator.
 **
 **  This program is intended to be run by nnrpd and handle usernames and
@@ -57,10 +56,10 @@
 #endif
 
 #if HAVE_PAM
-# if HAVE_PAM_PAM_APPL_H
-#  include <pam/pam_appl.h>
-# else
+# if HAVE_SECURITY_PAM_APPL_H
 #  include <security/pam_appl.h>
+# else
+#  include <pam/pam_appl.h>
 # endif
 #endif
 
@@ -85,7 +84,7 @@
 */
 #if HAVE_PAM
 static int
-pass_conv(int num_msg, const struct pam_message **msgm UNUSED,
+pass_conv(int num_msg, PAM_CONST struct pam_message **msgm UNUSED,
           struct pam_response **response, void *appdata_ptr)
 {
     int i;
@@ -316,6 +315,7 @@ main(int argc, char *argv[])
     enum authtype { AUTH_NONE, AUTH_SHADOW, AUTH_FILE, AUTH_DBM };
 
     int opt;
+    const char *hash;
     enum authtype type = AUTH_NONE;
     bool wantgroup = false;
     const char *filename = NULL;
@@ -410,7 +410,8 @@ main(int argc, char *argv[])
 
     if (password == NULL)
         die("user %s unknown", authinfo->username);
-    if (strcmp(password, crypt(authinfo->password, password)) != 0)
+    hash = crypt(authinfo->password, password);
+    if (hash == NULL || strcmp(password, hash) != 0)
         die("invalid password for user %s", authinfo->username);
 
     /* The password matched. */
