@@ -1,17 +1,20 @@
-/*  $Id: radius.c 8921 2010-01-22 23:33:08Z iulius $
-**
+/*
 **  Authenticate a user against a remote radius server.
 */
 
 #include "config.h"
 #include "clibrary.h"
-#include "portable/time.h"
 #include <ctype.h>
 #include <errno.h>
 #include <fcntl.h>
 #include <netdb.h>
 #include <signal.h>
 
+#ifdef HAVE_SYS_TIME_H
+# include <sys/time.h>
+#endif
+#include <time.h>
+
 /* Needed on AIX 4.1 to get fd_set and friends. */
 #if HAVE_SYS_SELECT_H
 # include <sys/select.h>
@@ -104,7 +107,6 @@ static int read_config(char *authfile, rad_config_t *radconf)
     rad_config_t *radconfig=NULL;
     CONFFILE *file;
     CONFTOKEN *token;
-    char *server;
     int type;
     char *iter;
 
@@ -118,7 +120,6 @@ static int read_config(char *authfile, rad_config_t *radconf)
           die("expected server keyword on line %d", file->lineno);
 	if ((token = CONFgettoken(0, file)) == NULL)
           die("expected server name on line %d", file->lineno);
-	server = xstrdup(token->name);
 	if ((token = CONFgettoken(radtoks, file)) == NULL 
 	    || token->type != RADlbrace)
           die("expected { on line %d", file->lineno);
@@ -211,14 +212,14 @@ static int read_config(char *authfile, rad_config_t *radconf)
 #define RAD_NAS_IP_ADDRESS      4       /* IP address */
 #define RAD_NAS_PORT            5       /* Integer */
 
-static void req_copyto (auth_req to, sending_t *from)
+static void req_copyto (auth_req *to, sending_t *from)
 {
-    to = from->req;
+    *to = from->req;
 }
 
-static void req_copyfrom (sending_t *to, auth_req from)
+static void req_copyfrom (sending_t *to, auth_req *from)
 {
-    to->req = from;
+    to->req = *from;
 }
 
 static int rad_auth(rad_config_t *radconfig, char *uname, char *pass)
@@ -267,7 +268,7 @@ static int rad_auth(rad_config_t *radconfig, char *uname, char *pass)
 	sreq->next = new;
 	sreq = sreq->next;
       }
-      req_copyto(req, sreq);
+      req_copyto(&req, sreq);
   
       /* first, build the sockaddrs */
       memset(&sinl, '\0', sizeof(sinl));
@@ -392,7 +393,7 @@ static int rad_auth(rad_config_t *radconfig, char *uname, char *pass)
       sreq->reqlen = req.length;
       req.length = htons(req.length);
 
-      req_copyfrom(sreq, req);
+      req_copyfrom(sreq, &req);
 
       /* Go to the next record in the list */
       config = config->next;
@@ -415,7 +416,7 @@ static int rad_auth(rad_config_t *radconfig, char *uname, char *pass)
     for(done = 0; authtries > 0 && !done; authtries--) {
       for (config = radconfig, sreq = reqtop; sreq != NULL && !done;
 	   config = config->next, sreq = sreq->next){
-	req_copyto(req, sreq);
+	req_copyto(&req, sreq);
 
 	/* send out the packet and wait for reply. */
 	if (sendto(sock, (char *)&req, sreq->reqlen, 0, 
@@ -479,10 +480,9 @@ static int rad_auth(rad_config_t *radconfig, char *uname, char *pass)
 	}
 	/* FINALLY!  Got back a known-good packet.  See if we're in. */
 	close(sock);
+        done = 1;
+        req_copyfrom(sreq, &req);
 	return (req.code == PW_AUTHENTICATION_ACK) ? 0 : -1;
-	done = 1;
-	req_copyfrom(sreq, req);
-	break;
       }
     }
     if (authtries == 0)
