@@ -1,5 +1,4 @@
-/*  $Id: buffchan.c,v 1.29 2003/01/19 22:56:32 rra Exp $
-**
+/*
 **  Buffered file exploder for innd.
 */
 
@@ -9,12 +8,13 @@
 #include <errno.h>
 #include <signal.h>
 #include <sys/stat.h>
+#include <time.h>
 
 #include "inn/innconf.h"
 #include "inn/messages.h"
 #include "inn/qio.h"
-#include "libinn.h"
-#include "paths.h"
+#include "inn/libinn.h"
+#include "inn/paths.h"
 #include "map.h"
 
 /*
@@ -64,7 +64,7 @@ static int	CloseSeconds;
 static int	FlushSeconds;
 static sig_atomic_t	GotInterrupt;
 static SITEHASH	SITEtable[SITE_SIZE];
-static TIMEINFO	Now;
+static time_t	Now;
 
 
 /*
@@ -121,10 +121,8 @@ SITEcloseall(void)
 */
 static void SITEopen(SITE *sp)
 {
-    int			e;
-
     if ((sp->F = xfopena(sp->Filename)) == NULL
-     && ((e = errno) != EACCES || chmod(sp->Filename, 0644) < 0
+     && (errno != EACCES || chmod(sp->Filename, 0644) < 0
       || (sp->F = xfopena(sp->Filename)) == NULL)) {
         syswarn("%s cannot fopen %s", sp->Name, sp->Filename);
 	if ((sp->F = fopen("/dev/null", "w")) == NULL)
@@ -140,8 +138,8 @@ static void SITEopen(SITE *sp)
     /* Reset all counters. */
     sp->FlushLines = 0;
     sp->CloseLines = 0;
-    sp->LastFlushed = Now.time;
-    sp->LastClosed = Now.time;
+    sp->LastFlushed = Now;
+    sp->LastClosed = Now;
     sp->Dropped = false;
 }
 
@@ -246,20 +244,20 @@ SITEwrite(char *name, char *text, size_t len)
 	SITEflush(sp);
 	return;
     }
-    if (CloseSeconds && sp->LastClosed + CloseSeconds < Now.time) {
+    if (CloseSeconds && sp->LastClosed + CloseSeconds < Now) {
 	SITEflush(sp);
 	return;
     }
     if (FlushEvery && ++(sp->FlushLines) >= FlushEvery) {
 	if (fflush(sp->F) == EOF || ferror(sp->F))
             syswarn("%s cannot flush %s", sp->Name, sp->Filename);
-	sp->LastFlushed = Now.time;
+	sp->LastFlushed = Now;
 	sp->FlushLines = 0;
     }
-    else if (FlushSeconds && sp->LastFlushed + FlushSeconds < Now.time) {
+    else if (FlushSeconds && sp->LastFlushed + FlushSeconds < Now) {
 	if (fflush(sp->F) == EOF || ferror(sp->F))
             syswarn("%s cannot flush %s", sp->Name, sp->Filename);
-	sp->LastFlushed = Now.time;
+	sp->LastFlushed = Now;
 	sp->FlushLines = 0;
     }
 }
@@ -316,7 +314,7 @@ Process(char *p)
 /*
 **  Mark that we got a signal; let two signals kill us.
 */
-static RETSIGTYPE
+static void
 CATCHinterrupt(int s)
 {
     GotInterrupt = true;
@@ -344,7 +342,7 @@ main(int ac, char *av[])
     /* Set defaults. */
     if (!innconf_read(NULL))
         exit(1);
-    ERRLOG = concatpath(innconf->pathlog, _PATH_ERRLOG);
+    ERRLOG = concatpath(innconf->pathlog, INN_PATH_ERRLOG);
     Directory = NULL;
     Fields = 1;
     Format = NULL;
@@ -378,7 +376,7 @@ main(int ac, char *av[])
 	case 'd':
 	    Directory = optarg;
 	    if (Format == NULL)
-		Format =xstrdup("%s");
+                Format = xstrdup("%s");
 	    break;
 	case 'f':
 	    Fields = atoi(optarg);
@@ -408,7 +406,6 @@ main(int ac, char *av[])
 	    break;
 	}
     ac -= optind;
-    av += optind;
     if (ac)
 	die("usage error");
 
@@ -431,7 +428,6 @@ main(int ac, char *av[])
 	    }
 	    if (QIOtoolong(qp)) {
                 warn("long line");
-		QIOread(qp);
 		continue;
 	    }
 
@@ -457,10 +453,8 @@ main(int ac, char *av[])
 	*p++ = '\n';
 	i = p - line;
 
-	if (GetTimeInfo(&Now) < 0) {
-            syswarn("cannot get time");
-	    break;
-	}
+        /* Update the current time. */
+        Now = time(NULL);
 
 	/* Rest of the line is space-separated list of filenames. */
 	for (; *p; p = next) {
