@@ -1,5 +1,4 @@
-/*  $Id: inndf.c 9714 2014-09-21 12:55:56Z iulius $
-**
+/*
 **  Reports free kilobytes (not disk blocks) or free inodes.
 **
 **  Written by Ian Dickinson <idickins@fore.com>
@@ -328,11 +327,11 @@ main(int argc, char *argv[])
        that query. */
     if (ovused) {
         used = overview_free_space(overview);
-        if (used == -1)
+        if (used < 0)
             printf("Space used is meaningless for the %s method\n",
                    innconf->ovmethod);
         else
-            printf("%.2f%% overview space used\n", used);
+            printf("%.2f%% overview space used\n", (double) used);
     }
     if (overview != NULL)
         overview_close(overview);
