@@ -1,5 +1,3 @@
-##  $Id: ihave.pl 8286 2009-01-11 12:07:11Z iulius $
-##
 ##  ihave control message handler.
 ##
 ##  Copyright 2001 by Marco d'Itri <md@linux.it>
@@ -19,35 +17,36 @@ use strict;
 
 sub control_ihave {
     my ($par, $sender, $replyto, $site, $action, $log, $approved,
-        $headers, $body) = @_;
+        $article) = @_;
+    my @body = split(/\r?\n/, $article->stringify_body);
 
     if ($action eq 'mail') {
         my $mail = sendmail("ihave by $sender");
-        print $mail map { s/^~/~~/; "$_\n" } @$body;
+        print $mail map { s/^~/~~/; "$_\n" } @body;
         close $mail or logdie('Cannot send mail: ' . $!);
     } elsif ($action eq 'log') {
         if ($log) {
-            logger($log, "ihave $sender", $headers, $body);
+            logger($log, "ihave $sender", $article);
         } else {
             logmsg("ihave $sender");
         }
     } elsif ($action eq 'doit') {
-        my $tempfile = "$inn::tmpdir/ihave.$$";
-        open(GREPHIST, "|grephistory -i > $tempfile")
+        my $tempfile = "$INN::Config::tmpdir/ihave.$$";
+        open(GREPHIST, "| $INN::Config::newsbin/grephistory -i > $tempfile")
             or logdie('Cannot run grephistory: ' . $!);
-	foreach (@$body) {
+	foreach (@body) {
             print GREPHIST "$_\n";
         }
         close GREPHIST;
 
         if (-s $tempfile) {
-            open(INEWS, "| $inn::inews -h")
+            open(INEWS, "| $INN::Config::inews -h")
                 or logdie('Cannot run inews: ' . $!);
             print INEWS "Newsgroups: to.$site\n"
-               . "Subject: cmsg sendme $inn::pathhost\n"
-               . "Control: sendme $inn::pathhost\n\n";
+               . "Subject: cmsg sendme $INN::Config::pathhost\n"
+               . "Control: sendme $INN::Config::pathhost\n\n";
             open(TEMPFILE, $tempfile) or logdie("Cannot open $tempfile: $!");
-            print INEWS $_ while <TEMPFILE>;  
+            print INEWS $_ while <TEMPFILE>;
             close INEWS or die $!;
             close TEMPFILE;
         }
