@@ -1,5 +1,4 @@
-/*  $Id: fastrm.c 9845 2015-05-02 16:32:08Z iulius $
-**
+/*
 **  Delete a list of filenames or tokens from stdin.
 **
 **  Originally written by <kre@munnari.oz.au> (to only handle files).
@@ -273,7 +272,6 @@ process_line(QIOSTATE *qp, int *queued, int *deleted)
                     dlen = strlen(line);
                     dir = xstrdup(line);
                 } else {
-                    p = line;
                     dlen = -1;
                     dir = NULL;
                 }
@@ -623,7 +621,7 @@ bad_path(const char *p)
 
 /*
 **  Main routine.  Parse options, initialize the storage manager, and
-**  initalize various global variables, and then go into a loop calling
+**  initialize various global variables, and then go into a loop calling
 **  process_line and unlink_filelist as needed.
 */
 int
