@@ -1,14 +1,13 @@
-/*  $Id: makehistory.c 8409 2009-04-11 22:13:41Z iulius $
-**
+/*
 **  Rebuild history/overview databases.
 */
 
 #include "config.h"
 #include "clibrary.h"
-#include "portable/wait.h"
 #include <assert.h>
 #include <errno.h>
 #include <syslog.h>
+#include <sys/wait.h>
 #include <time.h>
 
 #include "inn/buffer.h"
@@ -24,8 +23,28 @@
 #include "inn/vector.h"
 #include "inn/wire.h"
 
+/*
+**  If we have getloadavg, include the appropriate header file.  Otherwise,
+**  just assume that we always have a load of 0.
+*/
+#if HAVE_GETLOADAVG
+# if HAVE_SYS_LOADAVG_H
+#  include <sys/loadavg.h>
+# endif
+#else
+static int
+getloadavg(double loadavg[], int nelem)
+{
+    int i;
+
+    for (i = 0; i < nelem && i < 3; i++)
+        loadavg[i] = 0;
+    return i;
+}
+#endif
+
 static const char usage[] = "\
-Usage: makehistory [-abFIOSx] [-f file] [-l count] [-s size] [-T tmpdir]\n\
+Usage: makehistory [-abFIOSx] [-f file] [-l count] [-L load] [-s size] [-T tmpdir]\n\
 \n\
     -a          open output history file in append mode\n\
     -b          delete bad articles from spool\n\
@@ -33,6 +52,7 @@ Usage: makehistory [-abFIOSx] [-f file] [-l count] [-s size] [-T tmpdir]\n\
     -f file     write history entries to file (default $pathdb/history)\n\
     -I          do not create overview for articles numbered below lowmark\n\
     -l count    size of overview updates (default 10000)\n\
+    -L load     pause when load average exceeds threshold\n\
     -O          create overview entries for articles\n\
     -S          write overview data to standard output\n\
     -s size     size new history database for approximately size entries\n\
@@ -74,6 +94,7 @@ bool WriteStdout = false;
 static char             BYTES[] = "Bytes";
 static char             DATE[] = "Date";
 static char             EXPIRES[] = "Expires";
+static char             INJECTIONDATE[] = "Injection-Date";
 static char             LINES[] = "Lines";
 static char		MESSAGEID[] = "Message-ID";
 static char		XREF[] = "Xref";
@@ -83,6 +104,7 @@ static size_t		ARTfieldsize;
 static ARTOVERFIELD     *Bytesp = (ARTOVERFIELD *)NULL;
 static ARTOVERFIELD     *Datep = (ARTOVERFIELD *)NULL;
 static ARTOVERFIELD     *Expp = (ARTOVERFIELD *)NULL;
+static ARTOVERFIELD     *InjectionDatep = (ARTOVERFIELD *)NULL;
 static ARTOVERFIELD     *Linesp = (ARTOVERFIELD *)NULL;
 static ARTOVERFIELD     *Msgidp = (ARTOVERFIELD *)NULL;
 static ARTOVERFIELD     *Xrefp = (ARTOVERFIELD *)NULL;
@@ -130,6 +152,7 @@ FlushOverTmpFile(void)
     char temp[SMBUF];
     char *SortedTmpPath;
     int i, pid, fd;
+    float f;
     TOKEN token;
     QIOSTATE *qp;
     int count;
@@ -190,7 +213,10 @@ FlushOverTmpFile(void)
     if (fd < 0) {
         syswarn("cannot create temporary file");
         OVclose();
-        Fork ? _exit(1) : exit(1);
+        if (Fork)
+            _exit(1);
+        else
+            exit(1);
     }
     close(fd);
     snprintf(temp, sizeof(temp), "exec %s -T %s -t'%c' -o %s %s",
@@ -200,8 +226,11 @@ FlushOverTmpFile(void)
     if (i != 0) {
         syswarn("cannot sort temporary overview file (%s exited %d)",
                 INN_PATH_SORT, i);
-	OVclose();
-	Fork ? _exit(1) : exit(1);
+        OVclose();
+        if (Fork)
+            _exit(1);
+        else
+            exit(1);
     }
 
     /* don't need old path anymore. */
@@ -212,8 +241,11 @@ FlushOverTmpFile(void)
     /* read sorted lines. */
     if ((qp = QIOopen(SortedTmpPath)) == NULL) {
         syswarn("cannot open sorted overview file %s", SortedTmpPath);
-	OVclose();
-	Fork ? _exit(1) : exit(1);
+        OVclose();
+        if (Fork)
+            _exit(1);
+        else
+            exit(1);
     }
 
     for (count = 1; ; ++count) {
@@ -255,19 +287,25 @@ FlushOverTmpFile(void)
 	}
 	token = TextToToken(q);
 	if (OVadd(token, r, strlen(r), arrived, expires) == OVADDFAILED) {
-	    if (OVctl(OVSPACE, (void *)&i) && i == OV_NOSPACE) {
+	    if (OVctl(OVSPACE, (void *)&f) && (int)(f+0.01f) == OV_NOSPACE) {
                 warn("no space left for overview");
-		OVclose();
-		Fork ? _exit(1) : exit(1);
-	    }
+                OVclose();
+                if (Fork)
+                    _exit(1);
+                else
+                    exit(1);
+            }
             warn("cannot write overview data \"%.40s\"", q);
 	}
     }
     /* Check for errors and close. */
     if (QIOerror(qp)) {
         syswarn("cannot read sorted overview file %s", SortedTmpPath);
-	OVclose();
-	Fork ? _exit(1) : exit(1);
+        OVclose();
+        if (Fork)
+            _exit(1);
+        else
+            exit(1);
     }
     QIOclose(qp);
     /* unlink sorted tmp file */
@@ -291,7 +329,8 @@ WriteOverLine(TOKEN *token, const char *xrefs, int xrefslen,
 {
     char temp[SMBUF];
     const char *p, *q, *r;
-    int i, fd;
+    int fd;
+    float f;
 
     /* If WriteStdout is set, just print the overview information to standard
        output and return. */
@@ -313,7 +352,7 @@ WriteOverLine(TOKEN *token, const char *xrefs, int xrefslen,
                 sysdie("writing overview failed");
 	    fputc('\n', Overchan);
 	} else if (OVadd(*token, overdata, overlen, arrived, expires) == OVADDFAILED) {
-	    if (OVctl(OVSPACE, (void *)&i) && i == OV_NOSPACE) {
+	    if (OVctl(OVSPACE, (void *)&f) && (int)(f+0.01f) == OV_NOSPACE) {
                 warn("no space left for overview");
 		OVclose();
 		exit(1);
@@ -394,7 +433,7 @@ static void
 ARTreadschema(bool Overview)
 {
     const struct cvector        *standardoverview;
-    const struct vector         *extraoverview;
+    struct vector               *extraoverview;
     ARTOVERFIELD                *fp;
     unsigned int                i;
 
@@ -444,10 +483,15 @@ ARTreadschema(bool Overview)
             if (strncasecmp(extraoverview->strings[i],
                             EXPIRES, strlen(EXPIRES)) == 0)
                 Expp = fp;
+            if (strncasecmp(extraoverview->strings[i],
+                            INJECTIONDATE, strlen(INJECTIONDATE)) == 0)
+                InjectionDatep = fp;
             fp++;
         }
 
 	ARTfieldsize = fp - ARTfields;
+        vector_free(extraoverview);
+
     }
     if (Bytesp == (ARTOVERFIELD *)NULL)
         Missfieldsize++;
@@ -455,10 +499,14 @@ ARTreadschema(bool Overview)
         Missfieldsize++;
     if (Expp == (ARTOVERFIELD *)NULL)
         Missfieldsize++;
+    if (InjectionDatep == (ARTOVERFIELD *)NULL)
+        Missfieldsize++;
     if (Linesp == (ARTOVERFIELD *)NULL)
         Missfieldsize++;
     if (Msgidp == (ARTOVERFIELD *)NULL)
 	Missfieldsize++;
+    /* The fields in Missfields are not used in the overview (because they were not
+     * set before), so the values do not matter much (especially for NeedHeadername). */
     if (Missfieldsize > 0) {
 	Missfields = xmalloc(Missfieldsize * sizeof(ARTOVERFIELD));
         fp = Missfields;
@@ -481,7 +529,7 @@ ARTreadschema(bool Overview)
             Datep = fp++;
         }
         if (Expp == (ARTOVERFIELD *)NULL) {
-            fp->NeedHeadername = false;
+            fp->NeedHeadername = true;
             fp->Headername = xstrdup(EXPIRES);
             fp->HeadernameLength = strlen(EXPIRES);
             fp->Header = (char *)NULL;
@@ -489,6 +537,15 @@ ARTreadschema(bool Overview)
             fp->HeaderLength = 0;
             Expp = fp++;
         }
+        if (InjectionDatep == (ARTOVERFIELD *)NULL) {
+            fp->NeedHeadername = true;
+            fp->Headername = xstrdup(INJECTIONDATE);
+            fp->HeadernameLength = strlen(INJECTIONDATE);
+            fp->Header = (char *)NULL;
+            fp->HasHeader = false;
+            fp->HeaderLength = 0;
+            InjectionDatep = fp++;
+        }
         if (Linesp == (ARTOVERFIELD *)NULL) {
             fp->NeedHeadername = false;
             fp->Headername = xstrdup(LINES);
@@ -508,7 +565,7 @@ ARTreadschema(bool Overview)
             Msgidp = fp++;
         }
         if (Overview && Xrefp == (ARTOVERFIELD *)NULL) {
-	    fp->NeedHeadername = false;
+	    fp->NeedHeadername = true;
 	    fp->Headername = xstrdup(XREF);
 	    fp->HeadernameLength = strlen(XREF);
 	    fp->Header = (char *)NULL;
@@ -563,7 +620,7 @@ DoArt(ARTHANDLE *art)
 	}
     }
     for (fp = ARTfields, i = 0; i < ARTfieldsize; i++, fp++) {
-        fp->Header = wire_findheader(art->data, art->len, fp->Headername);
+        fp->Header = wire_findheader(art->data, art->len, fp->Headername, false);
 
         /* Someone managed to break their server so that they were appending
            multiple Xref headers, and INN had a bug where it wouldn't notice
@@ -579,7 +636,7 @@ DoArt(ARTHANDLE *art)
                     break;
                 next++;
                 left = art->len - (next - art->data);
-                next = wire_findheader(next, left, fp->Headername);
+                next = wire_findheader(next, left, fp->Headername, false);
                 if (next != NULL)
                     fp->Header = next;
             }
@@ -638,7 +695,7 @@ DoArt(ARTHANDLE *art)
     }
     if (Missfieldsize > 0) {
 	for (fp = Missfields, i = 0; i < Missfieldsize; i++, fp++) {
-            fp->Header = wire_findheader(art->data, art->len, fp->Headername);
+            fp->Header = wire_findheader(art->data, art->len, fp->Headername, false);
             if (fp->Header != NULL) {
 		fp->HasHeader = true;
                 p = wire_endheader(fp->Header, art->data + art->len - 1);
@@ -671,10 +728,8 @@ DoArt(ARTHANDLE *art)
         }
     }
 
-    MessageID = (char *)NULL;
     Arrived = art->arrived;
     Expires = 0;
-    Posted = 0;
 
     if (!Msgidp->HasHeader) {
         warn("no Message-ID header in %s", TokenToText(*art->token));
@@ -701,20 +756,30 @@ DoArt(ARTHANDLE *art)
      * newer than start time of makehistory. 
      */
 
-    if (!Datep->HasHeader) {
+    if (!InjectionDatep->HasHeader && !Datep->HasHeader) {
 	Posted = Arrived;
     } else {
-        buffer_set(&buffer, Datep->Header, Datep->HeaderLength);
-        buffer_append(&buffer, NUL, 1);
-        Posted = parsedate_rfc2822_lax(buffer.data);
-        if (Posted == (time_t) -1)
-	    Posted = Arrived;
+        if (InjectionDatep->HasHeader) {
+            buffer_set(&buffer, InjectionDatep->Header, InjectionDatep->HeaderLength);
+            buffer_append(&buffer, NUL, 1);
+            Posted = parsedate_rfc5322_lax(buffer.data);
+            /* If parsing failed for Injection-Date:, take the arrival time.
+             * Do not look at the Date: header (though we could). */
+            if (Posted == (time_t) -1)
+                Posted = Arrived;
+        } else {
+            buffer_set(&buffer, Datep->Header, Datep->HeaderLength);
+            buffer_append(&buffer, NUL, 1);
+            Posted = parsedate_rfc5322_lax(buffer.data);
+            if (Posted == (time_t) -1)
+                Posted = Arrived;
+        }
     }
 
     if (Expp->HasHeader) {
         buffer_set(&buffer, Expp->Header, Expp->HeaderLength);
         buffer_append(&buffer, NUL, 1);
-        Expires = parsedate_rfc2822_lax(buffer.data);
+        Expires = parsedate_rfc5322_lax(buffer.data);
         if (Expires == (time_t) -1)
             Expires = 0;
     }
@@ -810,6 +875,8 @@ main(int argc, char **argv)
 {
     ARTHANDLE *art = NULL;
     bool AppendMode;
+    int LoadAverage;
+    double load[1];
     int i;
     bool val;
     char *HistoryDir;
@@ -837,9 +904,10 @@ main(int argc, char **argv)
     DoOverview = false;
     Fork = false;
     AppendMode = false;
+    LoadAverage = 0;
     NoHistory = false;
 
-    while ((i = getopt(argc, argv, "abFf:Il:OSs:T:x")) != EOF) {
+    while ((i = getopt(argc, argv, "abFf:Il:L:OSs:T:x")) != EOF) {
 	switch(i) {
 	case 'a':
 	    AppendMode = true;
@@ -859,6 +927,9 @@ main(int argc, char **argv)
 	case 'l':
 	    OverTmpSegSize = atoi(optarg);
 	    break;
+        case 'L':
+            LoadAverage = atoi(optarg);
+            break;
 	case 'O':
 	    DoOverview = true;
 	    break;
@@ -883,7 +954,6 @@ main(int argc, char **argv)
 	}
     }
     argc -= optind;
-    argv += optind;
     if (argc) {
         fprintf(stderr, "%s", usage);
         exit(1);
@@ -956,7 +1026,7 @@ main(int argc, char **argv)
 
     /*
      * Scan the entire spool, nuke any bad arts if needed, and process each
-     * article.
+     * article.  We take a break when the load is too high.
      */
 	
     while ((art = SMnext(art, RETR_ALL)) != NULL) {
@@ -965,7 +1035,15 @@ main(int argc, char **argv)
 		SMcancel(*art->token);
 	    continue;
 	}
+
 	DoArt(art);
+
+        if (LoadAverage > 0) {
+            while (getloadavg(load, 1) > 0 &&
+                   (int) (load[0]) >= LoadAverage) {
+                sleep(1);
+            }
+        }
     }
 
     if (!NoHistory) {
