@@ -1,5 +1,4 @@
-/*  $Id: prunehistory.c 6124 2003-01-14 06:03:29Z rra $
-**
+/*
 **  Prune file names from history file.
 */
 
@@ -11,9 +10,10 @@
 #include "inn/history.h"
 #include "inn/innconf.h"
 #include "inn/messages.h"
-#include "libinn.h"
-#include "paths.h"
+#include "inn/libinn.h"
+#include "inn/paths.h"
 
+static void Usage(void) __attribute__ ((__noreturn__));
 
 /*
 **  Print usage message and exit.
@@ -21,7 +21,7 @@
 static void
 Usage(void)
 {
-    fprintf(stderr, "Usage:  prunehistory [-p] [-f file] [input]\n");
+    fprintf(stderr, "Usage:  prunehistory [-p] [-f file]\n");
     exit(1);
 }
 
@@ -45,7 +45,7 @@ main(int ac, char *av[])
     if (!innconf_read(NULL))
         exit(1);
 
-    History = concatpath(innconf->pathdb, _PATH_HISTORY);
+    History = concatpath(innconf->pathdb, INN_PATH_HISTORY);
     Passing = false;
 
     /* Parse JCL. */
@@ -62,7 +62,6 @@ main(int ac, char *av[])
 	    break;
 	}
     ac -= optind;
-    av += optind;
     if (ac) {
 	Usage();
 	rc = 1;
