@@ -1,15 +1,15 @@
-/*  $Id: feedone.c 7641 2007-08-10 07:56:30Z iulius $
-**
+/*
 **  Connect to the NNTP server and feed one article.
 */
 
 #include "config.h"
 #include "clibrary.h"
 #include <errno.h>
+#include <time.h>
 
-#include "inn/messages.h"
 #include "inn/libinn.h"
-#include "nntp.h"
+#include "inn/messages.h"
+#include "inn/nntp.h"
 
 
 static FILE	*FromServer;
@@ -27,7 +27,7 @@ GetFromServer(buff, size, text)
     char	*text;
 {
     if (fgets(buff, size, FromServer) == NULL)
-        sysdie("s", text);
+        sysdie("%s", text);
     if (Tracing)
 	printf("S: %s", buff);
 }
@@ -83,7 +83,6 @@ main(ac, av)
     bool	PostMode;
 
     /* Set defaults. */
-    mesgid[0] = '\0';
     PostMode = false;
     message_program_name = "feedone";
 
@@ -122,7 +121,7 @@ main(ac, av)
     /* Scan for the message-id. */
     if (mesgid == NULL) {
 	while (fgets(buff, sizeof buff, F) != NULL)
-	    if (strncasecmp(buff, MESGIDHDR, strlen(MESGIDHDR)) == 0) {
+	    if (strncmp(buff, MESGIDHDR, strlen(MESGIDHDR)) == 0) {
 		if ((p = strchr(buff, '<')) == NULL
                  || (q = strchr(p, '>')) == NULL)
                     die("bad message ID line");
@@ -163,7 +162,7 @@ main(ac, av)
     /* Send the file over. */
     fseeko(F, 0, SEEK_SET);
     while (fgets(buff, sizeof buff, F) != NULL) {
-	if (strncasecmp(buff, MESGIDHDR, strlen(MESGIDHDR)) == 0) {
+	if (strncmp(buff, MESGIDHDR, strlen(MESGIDHDR)) == 0) {
 	    fprintf(ToServer, "%s %s\r\n", MESGIDHDR, mesgid);
 	    continue;
 	}
