@@ -6,10 +6,8 @@
 #include "config.h"
 #include "clibrary.h"
 #include "portable/mmap.h"
-#include "portable/time.h"
 #include "portable/setproctitle.h"
 #include "portable/socket.h"
-#include "portable/wait.h"
 #include <errno.h>
 #include <fcntl.h>
 #include <signal.h>
@@ -18,10 +16,17 @@
 #endif
 #include <syslog.h>
 
-#ifdef HAVE_UNIX_DOMAIN_SOCKETS
-# include <sys/un.h>
+#ifdef HAVE_SYS_TIME_H
+# include <sys/time.h>
 #endif
+#include <time.h>
 
+#ifdef HAVE_UNIX_DOMAIN_SOCKETS
+# include "portable/socket-unix.h"
+#endif
+#include <sys/wait.h>
+
+#include "inn/fdflag.h"
 #include "inn/innconf.h"
 #include "inn/messages.h"
 #include "inn/libinn.h"
@@ -32,7 +37,7 @@
 #include "../storage/ovdb/ovdb.h"
 #include "../storage/ovdb/ovdb-private.h"
 
-#ifndef USE_BERKELEY_DB
+#ifndef HAVE_BDB
 
 int
 main(int argc UNUSED, char **argv UNUSED)
@@ -40,7 +45,7 @@ main(int argc UNUSED, char **argv UNUSED)
     die("Berkeley DB support not compiled");
 }
 
-#else /* USE_BERKELEY_DB */
+#else /* HAVE_BDB */
 
 
 #define SELECT_TIMEOUT 15
@@ -274,7 +279,7 @@ process_cmd(struct reader *r)
 	case CMD_ARTINFO:
 	    r->state = STATE_READGROUP;
 	    if(cmd->grouplen == 0) {
-		/* shoudn't happen... */
+		/* shouldn't happen... */
 		r->mode = MODE_CLOSED;
 		close(r->fd);
 		free(r->buf);
@@ -380,7 +385,7 @@ newclient(int fd)
     struct reader *r;
     int i;
 
-    nonblocking(fd, 1);
+    fdflag_nonblocking(fd, 1);
 
     if(numreaders >= readertablen) {
     	readertablen += 50;
@@ -646,7 +651,7 @@ main(int argc, char *argv[])
     if(listensock < 0)
         sysdie("cannot create socket");
 
-    nonblocking(listensock, 1);
+    fdflag_nonblocking(listensock, 1);
 
     memset(&sa, 0, sizeof sa);
 #ifdef HAVE_UNIX_DOMAIN_SOCKETS
@@ -673,7 +678,7 @@ main(int argc, char *argv[])
 
     if(ret != 0)
         sysdie("cannot bind socket");
-    if(listen(listensock, MAXLISTEN) < 0)
+    if(listen(listensock, innconf->maxlisten) < 0)
         sysdie("cannot listen on socket");
 
     pidfile = concatpath(innconf->pathrun, OVDB_SERVER_PIDFILE);
@@ -742,4 +747,4 @@ main(int argc, char *argv[])
 }
 
 
-#endif /* USE_BERKELEY_DB */
+#endif /* HAVE_BDB */
