@@ -1,5 +1,4 @@
-/*  $Id: rnews.c 9855 2015-05-14 13:13:11Z iulius $
-**
+/*
 **  A front-end for InterNetNews.
 **
 **  Read UUCP batches and offer them up NNTP-style.  Because we may end
@@ -9,14 +8,15 @@
 
 #include "config.h"
 #include "clibrary.h"
-#include "portable/wait.h"
 #include <ctype.h>
 #include <dirent.h>
 #include <errno.h>
 #include <fcntl.h>
 #include <syslog.h>
 #include <sys/stat.h>
+#include <sys/wait.h>
 
+#include "inn/fdflag.h"
 #include "inn/innconf.h"
 #include "inn/libinn.h"
 #include "inn/messages.h"
@@ -33,7 +33,10 @@ typedef struct _HEADER {
 } HEADER;
 
 
-static bool	Verbose;
+static bool     additionalUnpackers = true;
+static bool     backupBad = false;
+static bool     logDuplicates = false;
+static bool     Verbose = false;
 static const char	*InputFile = "stdin";
 static char	*UUCPHost;
 static char	*PathBadNews = NULL;
@@ -163,37 +166,49 @@ static void
 Reject(const char *article, size_t length UNUSED, const char *reason,
        const char *arg)
 {
-#if	defined(DO_RNEWS_SAVE_BAD)
     char *filename;
     FILE *F;
     int fd;
-#endif	/* defined(DO_RNEWS_SAVE_BAD) */
 
+#if __GNUC__ > 4
+# pragma GCC diagnostic ignored "-Wformat-nonliteral"
+#endif
     notice(reason, arg);
+#if __GNUC__ > 4
+# pragma GCC diagnostic warning "-Wformat-nonliteral"
+#endif
     if (Verbose) {
 	fprintf(stderr, "%s: ", InputFile);
+#if __GNUC__ > 4
+# pragma GCC diagnostic ignored "-Wformat-nonliteral"
+#endif
 	fprintf(stderr, reason, arg);
+#if __GNUC__ > 4
+# pragma GCC diagnostic warning "-Wformat-nonliteral"
+#endif
 	fprintf(stderr, " [%.40s...]\n", article);
     }
 
-#if	defined(DO_RNEWS_SAVE_BAD)
-    filename = concat(PathBadNews, "/XXXXXX", (char *) 0);
-    fd = mkstemp(filename);
-    if (fd < 0) {
-        warn("cannot create temporary file");
-        return;
+    if (backupBad) {
+        filename = concat(PathBadNews, "/XXXXXX", (char *) 0);
+        fd = mkstemp(filename);
+        if (fd < 0) {
+            warn("cannot create temporary file");
+            free(filename);
+            return;
+        }
+        F = fdopen(fd, "w");
+        if (F == NULL) {
+            warn("cannot fdopen %s", filename);
+            free(filename);
+            return;
+        }
+        if (fwrite(article, 1, length, F) != length)
+            warn("cannot fwrite to %s", filename);
+        if (fclose(F) == EOF)
+            warn("cannot close %s", filename);
+        free(filename);
     }
-    F = fdopen(fd, "w");
-    if (F == NULL) {
-        warn("cannot fdopen %s", filename);
-	return;
-    }
-    if (fwrite(article, 1, length, F) != length)
-        warn("cannot fwrite to %s", filename);
-    if (fclose(F) == EOF)
-        warn("cannot close %s", filename);
-    free(filename);
-#endif	/* defined(DO_RNEWS_SAVE_BAD) */
 }
 
 
@@ -212,12 +227,7 @@ Process(char *article, size_t artlen)
     const char		*id = NULL;
     char                *msgid;
     char		buff[SMBUF];
-#if	defined(FILE_RNEWS_LOG_DUPS)
-    FILE		*F;
-#endif	/* defined(FILE_RNEWS_LOG_DUPS) */
-#if	!defined(DONT_RNEWS_LOG_DUPS)
     char		path[40];
-#endif	/* !defined(DONT_RNEWS_LOG_DUPS) */
 
     /* Empty article? */
     if (*article == '\0')
@@ -238,13 +248,13 @@ Process(char *article, size_t artlen)
 	    id = p;
 	    continue;
 	}
-#if	!defined(DONT_RNEWS_LOG_DUPS)
-	if (IS_PATH(hp)) {
-	    strlcpy(path, p, sizeof(path));
-	    if ((q = strchr(path, '\r')) != NULL)
-		*q = '\0';
-	}
-#endif	/* !defined(DONT_RNEWS_LOG_DUPS) */
+        if (logDuplicates) {
+            if (IS_PATH(hp)) {
+                strlcpy(path, p, sizeof(path));
+                if ((q = strchr(path, '\r')) != NULL)
+                    *q = '\0';
+            }
+        }
     }
 
     /* Send the NNTP "ihave" message. */
@@ -287,17 +297,9 @@ Process(char *article, size_t artlen)
     case NNTP_CONT_IHAVE:
 	break;
     case NNTP_FAIL_IHAVE_REFUSE:
-#if	defined(SYSLOG_RNEWS_LOG_DUPS)
-	*p = '\0';
-        notice("duplicate %s %s", id, path);
-#endif	/* defined(SYSLOG_RNEWS_LOG_DUPS) */
-#if	defined(FILE_RNEWS_LOG_DUPS)
-	if ((F = fopen(INN_PATH_RNEWS_DUP_LOG, "a")) != NULL) {
-	    *p = '\0';
-	    fprintf(F, "duplicate %s %s\n", id, path);
-	    fclose(F);
-	}
-#endif	/* defined(FILE_RNEWS_LOG_DUPS) */
+        if (logDuplicates) {
+            notice("duplicate %s %s", msgid, path);
+        }
         free(wirefmt);
 	return true;
     }
@@ -434,12 +436,13 @@ ReadBytecount(int fd, int artsize)
        seems like the best of a set of bad options; Reject would save the
        article into bad and then someone might reprocess it, leaving us with
        accepting the truncated version. */
-    for (p = article, left = artsize; left; p += i, left -= i)
-	if ((i = read(fd, p, left)) <= 0) {
-	    i = errno;
+    for (p = article, left = artsize; left; p += i, left -= i) {
+        i = read(fd, p, left);
+        if (i <= 0) {
             warn("cannot read, wanted %d got %d", artsize, artsize - left);
-	    return true;
-	}
+            return true;
+        }
+    }
     if (p[-1] != '\n') {
 	*p++ = '\n';
         artsize++;
@@ -482,10 +485,9 @@ ReadLine(char *p, int size, int fd)
 static bool
 UnpackOne(int *fdp, size_t *countp)
 {
-#if	defined(DO_RNEWSPROGS)
     char	path[(SMBUF * 2) + 1];
     char	*p;
-#endif	/* defined(DO_RNEWSPROGS) */
+    int         len;
     char	buff[SMBUF];
     const char *cargv[4];
     int		artsize;
@@ -493,7 +495,6 @@ UnpackOne(int *fdp, size_t *countp)
     int		gzip = 0;
     bool	HadCount;
     bool	SawCunbatch;
-    int		len;
 
     *countp = 0;
     for (SawCunbatch = false, HadCount = false; ; ) {
@@ -574,36 +575,39 @@ UnpackOne(int *fdp, size_t *countp)
 	    continue;
 	}
 
-#if	defined(DO_RNEWSPROGS)
-	cargv[0] = UNPACK;
-	cargv[1] = NULL;
-	/* Ignore any possible leading pathnames, to avoid trouble. */
-	if ((p = strrchr(&buff[3], '/')) != NULL)
-	    p++;
-	else
-	    p = &buff[3];
-	if (strchr(INN_PATH_RNEWSPROGS, '/') == NULL) {
-	    snprintf(path, sizeof(path), "%s/%s/%s", innconf->pathbin,
-                     INN_PATH_RNEWSPROGS, p);
-	    len = strlen(innconf->pathbin) + 1 + sizeof INN_PATH_RNEWSPROGS;
-	} else {
-	    snprintf(path, sizeof(path), "%s/%s", INN_PATH_RNEWSPROGS, p);
-	    len = sizeof INN_PATH_RNEWSPROGS;
-	}
-	for (p = &path[len]; *p; p++)
-	    if (ISWHITE(*p)) {
-		*p = '\0';
-		break;
-	    }
-	*fdp = StartChild(*fdp, path, cargv);
-	if (*fdp < 0)
-	    return false;
-	(*countp)++;
-	continue;
-#else
-        warn("bad_format unknown command %s", buff);
-	return false;
-#endif	/* defined(DO_RNEWSPROGS) */
+        if (additionalUnpackers) {
+            cargv[0] = UNPACK;
+            cargv[1] = NULL;
+            /* Ignore any possible leading pathnames, to avoid trouble. */
+            if ((p = strrchr(&buff[3], '/')) != NULL) {
+                p++;
+            } else {
+                p = &buff[3];
+            }
+            if (strchr(INN_PATH_RNEWSPROGS, '/') == NULL) {
+                snprintf(path, sizeof(path), "%s/%s/%s", innconf->pathbin,
+                         INN_PATH_RNEWSPROGS, p);
+                len = strlen(innconf->pathbin) + 1
+                    + sizeof(INN_PATH_RNEWSPROGS);
+            } else {
+                snprintf(path, sizeof(path), "%s/%s", INN_PATH_RNEWSPROGS, p);
+                len = sizeof(INN_PATH_RNEWSPROGS);
+            }
+            for (p = &path[len]; *p; p++)
+                if (ISWHITE(*p)) {
+                    *p = '\0';
+                    break;
+                }
+            *fdp = StartChild(*fdp, path, cargv);
+            if (*fdp < 0) {
+                return false;
+            }
+            (*countp)++;
+            continue;
+        } else {
+            warn("bad_format unknown command %s", buff);
+            return false;
+        }
     }
     return true;
 }
@@ -855,7 +859,14 @@ int main(int ac, char *av[])
        other setups where rnews might be setuid news or be run by other
        processes in the news group. */
     if (getuid() == 0 || geteuid() == 0) {
-        ensure_news_user(true);
+        uid_t uid;
+
+        /* Do not use ensure_news_user() because it will fail to deal
+         * with the case of rnews being setuid news. */
+        get_news_uid_gid(&uid, false, true);
+        if (setuid(uid) < 0) {
+            sysdie("failed to setuid");
+        }
     }
 
     if (!innconf_read(NULL))
@@ -869,11 +880,20 @@ int main(int ac, char *av[])
     /* Parse JCL. */
     fd = STDIN_FILENO;
     mode = '\0';
-    while ((i = getopt(ac, av, "h:P:NUvr:S:")) != EOF)
+    while ((i = getopt(ac, av, "abdh:NP:r:S:Uv")) != EOF)
 	switch (i) {
 	default:
 	    die("usage error");
 	    /* NOTRTEACHED */
+        case 'a':
+            additionalUnpackers = false;
+            break;
+        case 'b':
+            backupBad = true;
+            break;
+        case 'd':
+            logDuplicates = true;
+            break;
 	case 'h':
 	    UUCPHost = *optarg ? optarg : NULL;
 	    break;
@@ -932,8 +952,8 @@ int main(int ac, char *av[])
 			CantConnect(buff, mode, fd);
 	}
     }
-    close_on_exec(fileno(FromServer), true);
-    close_on_exec(fileno(ToServer), true);
+    fdflag_close_exec(fileno(FromServer), true);
+    fdflag_close_exec(fileno(ToServer), true);
 
     /* Execute the command. */
     if (mode == 'U')
