@@ -1,5 +1,4 @@
-/*  $Id: sys2nf.c 9024 2010-03-21 16:49:30Z iulius $
-**
+/*
 **  Read a C news "sys" file and split it up into a set of INN
 **  newsfeeds entries.  Also works with B news.
 **
@@ -309,12 +308,19 @@ main(ac, av)
 	DoSub(F, f2);
 	fprintf(F, "\\\n");
 	if (strcmp(f3, "n") == 0)
-	    fprintf(F, "\t:Tf,Wnm\\\n", f3);
+	    fprintf(F, "\t:Tf,Wnm\\\n");
 	else
 	    fprintf(F, "\t:HELP%s\\\n", f3);
 	fprintf(F, "\t:%s\n", f4);
-	if (ferror(F) || fclose(F) == EOF)
-	    perror(TEMPFILE), exit(1);
+        if (ferror(F) != 0) {
+            perror(TEMPFILE);
+            fclose(F);
+            exit(1);
+        }
+       if (fclose(F) == EOF) {
+            perror(TEMPFILE);
+            exit(1);
+        }
 
 	free(site);
 
