@@ -1,5 +1,4 @@
-/*  $Id: hisv6.c 8575 2009-08-18 13:53:54Z iulius $
-**
+/*
 **  History v6 implementation against the history API.
 **
 **  Copyright (c) 2001, Thus plc 
@@ -46,6 +45,7 @@
 #include "hisv6.h"
 #include "hisv6-private.h"
 #include "inn/dbz.h"
+#include "inn/fdflag.h"
 #include "inn/innconf.h"
 #include "inn/timer.h"
 #include "inn/qio.h"
@@ -78,9 +78,9 @@ hisv6_errloc(char *s, size_t line, off_t offset)
 {
     if (offset != -1) {
 	/* really we want an autoconf test for %ll/%L/%I64, sigh */
-	sprintf(s, "@%.0f", (double)offset);
+        snprintf(s, HISV6_MAX_LOCATION, "@%lu", (unsigned long)offset);
     } else {
-	sprintf(s, ":%lu", (unsigned long)line);
+        snprintf(s, HISV6_MAX_LOCATION, ":%lu", (unsigned long)line);
     }
 }
 
@@ -314,7 +314,7 @@ hisv6_reopen(struct hisv6 *h)
 	    hisv6_closefiles(h);
 	    goto fail;
 	}
-	close_on_exec(fileno(h->writefp), true);
+	fdflag_close_exec(fileno(h->writefp), true);
     }
 
     /* Open the history file for reading. */
@@ -324,7 +324,7 @@ hisv6_reopen(struct hisv6 *h)
 	hisv6_closefiles(h);
 	goto fail;
     }
-    close_on_exec(h->readfd, true);
+    fdflag_close_exec(h->readfd, true);
     
     /* if there's no current dbz owner, claim it here */
     if (hisv6_dbzowner == NULL) {
@@ -1133,7 +1133,7 @@ hisv6_expirecb(struct hisv6 *h, void *cookie, const HASH *hash,
 
     /* check if we've seen this message id already */
     if (hiscookie->new && dbzexists(*hash)) {
-	/* continue after duplicates, it serious, but not fatal */
+	/* continue after duplicates, it's serious, but not fatal */
 	hisv6_seterror(h, concat("duplicate message-id [",
 				 HashToText(*hash), "] in history ",
 				 hiscookie->new->histpath, NULL));
