@@ -1,5 +1,4 @@
-/*  $Id: confparse.h 6883 2004-05-24 22:42:07Z rra $
-**
+/*
 **  Configuration file parsing interface.
 */
 
@@ -50,7 +49,8 @@ const char *config_group_tag(struct config_group *);
    unchanged and that parameter wasn't set (or was set to an invalid value for
    the expected type). */
 bool config_param_boolean(struct config_group *, const char *, bool *);
-bool config_param_integer(struct config_group *, const char *, long *);
+bool config_param_signed_number(struct config_group *, const char *, long *);
+bool config_param_unsigned_number(struct config_group *, const char *, unsigned long *);
 bool config_param_real(struct config_group *, const char *, double *);
 bool config_param_string(struct config_group *, const char *, const char **);
 bool config_param_list(struct config_group *, const char *,
@@ -64,9 +64,11 @@ struct vector *config_params(struct config_group *);
    error at a particular parameter in a config_group and config_error_group
    reports an error at the definition of that group.  The error is reported
    using warn. */
-void config_error_group(struct config_group *, const char *format, ...);
+void config_error_group(struct config_group *, const char *format, ...)
+    __attribute__((__format__(printf, 2, 3)));
 void config_error_param(struct config_group *, const char *key,
-                        const char *format, ...);
+                        const char *format, ...)
+    __attribute__((__format__(printf, 3, 4)));
 
 /* Free all space allocated by the tree rooted at config_group.  One normally
    never wants to do this.  WARNING: This includes the storage allocated for
