@@ -1,5 +1,4 @@
-/*  $Id: innconf.h 9753 2014-12-01 20:26:38Z iulius $
-**
+/*
 **  inn.conf parser interface.
 **
 **  The interface to reading inn.conf configuration files and managing the
@@ -15,8 +14,8 @@
 /*
 **  This structure is organized in the same order as the variables contained
 **  in it are mentioned in the inn.conf documentation, and broken down into
-**  the same sections.  Note that due to the implementation, only three types
-**  of variables are permissible here:  char *, bool, struc vector *, long
+**  the same sections.  Note that due to the implementation, only five types
+**  of variables are permissible here:  char *, bool, struct vector *, long
 **  and unsigned long.
 */
 struct innconf {
@@ -26,9 +25,10 @@ struct innconf {
     char *mailcmd;              /* Command to send report/control type mail */
     char *mta;                  /* MTA for mailing to moderators, innmail */
     char *pathhost;             /* Entry for the Path: line */
-    char *server;               /* Default server to connect to */
     char *runasuser;            /* User to run under */
     char *runasgroup;           /* Group to run under */
+    char *server;               /* Default server to connect to */
+    struct vector *syntaxchecks; /* List of syntax checks to perform or not */
 
     /* Feed Configuration */
     unsigned long artcutoff;    /* Max accepted article age */
@@ -99,10 +99,11 @@ struct innconf {
     unsigned long keymaxwords;  /* Max count of interesting words */
 
     /* Posting */
-    bool addnntppostingdate;    /* Add NNTP-Posting-Date: to posts */
-    bool addnntppostinghost;    /* Add NNTP-Posting-Host: to posts */
+    bool addinjectiondate;      /* Add Injection-Date: to posts */
+    bool addinjectionpostingaccount; /* Add posting-account attribute to Injection-Info: to posts */
+    bool addinjectionpostinghost; /* Add posting-host attribute to Injection-Info: to posts */
     bool checkincludedtext;     /* Reject if too much included text */
-    char *complaints;           /* Address for X-Complaints-To: */
+    char *complaints;           /* Address for mail-complaints-to attribute to Injection-Info: */
     char *fromhost;             /* Host for the From: line */
     unsigned long localmaxartsize; /* Max article size of local postings */
     char *moderatormailer;      /* Default host to mail moderated articles */
@@ -121,18 +122,21 @@ struct innconf {
     unsigned long backoffpostslow; /* Lower time limit for slow posting */
     unsigned long backofftrigger;  /* Number of postings before triggered */
 
-    /* Reading and posting -- SSL and TLS support */
-#ifdef HAVE_SSL
+    /* Reading and posting -- TLS/SSL support */
+    /* Do not test HAVE_OPENSSL.  This relieves customers of /usr/include/inn
+     * from the need to guess whether INN was built with TLS/SSL support in
+     * order to get a header that matches the installed libraries.
+     */
     char *tlscafile;            /* Path to a certificate authority file */
     char *tlscapath;            /* Path to a directory of CA certificates */
-    char *tlscertfile;          /* Path to the SSL certificate to use */
+    char *tlscertfile;          /* Path to the TLS/SSL certificate to use */
     char *tlskeyfile;           /* Path to the key for the certificate */
     char *tlsciphers;           /* OpenSSL-style cipher string */
+    char *tlsciphers13;         /* OpenSSL-style cipher string for TLS 1.3 */
     bool tlscompression;        /* Turn TLS compression on/off */
     char *tlseccurve;           /* ECDH curve name */
     bool tlspreferserverciphers; /* Make server select the cipher */
     struct vector *tlsprotocols; /* List of supported TLS versions */
-#endif /* HAVE_SSL */
 
     /* Monitoring */
     bool doinnwatch;            /* Start innwatch from rc.news? */
@@ -158,8 +162,8 @@ struct innconf {
     bool logtrash;              /* Log unwanted newsgroups? */
     bool nnrpdoverstats;        /* Log overview statistics? */
     bool nntplinklog;           /* Put storage token into the log? */
-    unsigned long status;       /* Status file update interval */
     char *stathist;             /* Filename for history profiler outputs */
+    unsigned long status;       /* Status file update interval */
     unsigned long timer;        /* Performance monitoring interval */
 
     /* System Tuning */
@@ -172,6 +176,7 @@ struct innconf {
     unsigned long keepmmappedthreshold; /* Threshold for keeping mmap in buffindexed */
     unsigned long maxcmdreadsize; /* Max NNTP command read size used by innd */
     unsigned long maxforks;     /* Give up after this many fork failure. */
+    unsigned long maxlisten;    /* Max listen() backlog */
     long nicekids;              /* Child processes get niced to this */
     unsigned long nicenewnews;  /* If NEWNEWS command is used, nice to this */
     unsigned long nicennrpd;    /* nnrpd is niced to this */
