@@ -1,5 +1,4 @@
-/*  $Id: setproctitle.h 8838 2009-12-03 21:11:00Z iulius $
-**
+/*
 **  Set things up for setproctitle portably.
 **
 **  If the system supports setproctitle, we need to define away
@@ -11,6 +10,7 @@
 #define PORTABLE_SETPROCTITLE_H 1
 
 #include "config.h"
+#include "portable/macros.h"
 
 #if !HAVE_SETPROCTITLE || !HAVE_DECL_SETPROCTITLE
 void setproctitle(const char *format, ...)
