@@ -1,5 +1,4 @@
-/*  $Id: ng.c 6494 2003-10-20 01:12:50Z rra $
-**
+/*
 **  Routine for the in-core data structures for the active and newsfeeds
 **  files.
 */
@@ -10,7 +9,7 @@
 
 #include "inn/innconf.h"
 #include "innd.h"
-#include "ov.h"
+#include "inn/ov.h"
 
 
 /*
@@ -44,7 +43,7 @@ static NGHASH		NGHtable[NGH_SIZE];
 static int		NGHbuckets;
 static int		NGHcount;
 
-
+
 
 /*
 **  Sorting predicate for qsort call in NGparsefile.  Put newsgroups in
@@ -54,8 +53,10 @@ static int		NGHcount;
 static int
 NGcompare(const void *p1, const void *p2)
 {
-    return ((const NEWSGROUP **)p1)[0]->Last -	
-	   ((const NEWSGROUP **)p2)[0]->Last;
+    const NEWSGROUP *const *n1 = p1;
+    const NEWSGROUP *const *n2 = p2;
+
+    return n1[0]->Last - n2[0]->Last;
 }
 
 
@@ -321,13 +322,13 @@ NGsplit(char *p, int size, LISTBUFFER *list)
 /*
 **  Renumber a group.
 */
-static char		NORENUMBER[] = "%s cant renumber %s %s too wide";
-static char		RENUMBER[] = "%s renumber %s %s from %ld to %ld";
+#define NORENUMBER "%s cant renumber %s %s too wide"
+#define RENUMBER "%s renumber %s %s from %ld to %ld"
 
 bool
 NGrenumber(NEWSGROUP *ngp)
 {
-    int			low, high, count,flag;
+    int			low, high, count, flag;
     char	        *f2;
     char		*f3;
     char		*f4;
@@ -353,20 +354,23 @@ NGrenumber(NEWSGROUP *ngp)
 	    LogName, MaxLength(start, start));
 	return false;
     }
-    himark = atol(f2);
-    lomark = himark + 1;
-    /* note these will be the low and himarks if the group turns out to be empty. */
 
     /* Check overview data for the group. */
     if (!OVgroupstats(ngp->Name, &low, &high, &count, &flag)) return false;
     if (count != 0) {
-	/* non-empty group, so set low/himarks from overview. */
+	/* Non-empty group, so set low/high water marks from overview. */
 	lomark = low;
 	himark = high;
+    } else {
+        /* Empty group. */
+        himark = high;
+        lomark = himark + 1;
     }
     l = atol(f2);
     if (himark > l) {
 	syslog(L_NOTICE, RENUMBER, LogName, ngp->Name, "hi", l, himark);
+        /* f3 - f2 - 1 gives the length of the full high water mark
+         * in the active file. */
 	if (!FormatLong(f2, himark, f3 - f2 - 1)) {
 	    syslog(L_ERROR, NORENUMBER, LogName, ngp->Name, "hi");
 	    return false;
@@ -381,6 +385,9 @@ NGrenumber(NEWSGROUP *ngp)
     if (lomark != l) {
 	if (lomark < l)
 	    syslog(L_NOTICE, RENUMBER, LogName, ngp->Name, "lo", l, lomark);
+        /* f4 - f3 gives the length of the full low water mark
+         * in the active file.
+         * f4 points at the space ' ' before the newsgroup flag. */
 	if (!FormatLong(f3, lomark, f4 - f3)) {
 	    syslog(L_ERROR, NORENUMBER, LogName, ngp->Name, "lo");
 	    return false;
@@ -391,10 +398,9 @@ NGrenumber(NEWSGROUP *ngp)
 }
 
 /*
- * Set the low article count for the given group.
- * Like NGrenumber(), but we don't scan the spool,
- * and the himark is ignored.
- */
+** Set the low article count for the given group.
+** Like NGrenumber(), but we don't query the overview database.
+*/
 bool
 NGlowmark(NEWSGROUP *ngp, long lomark)
 {
@@ -411,6 +417,20 @@ NGlowmark(NEWSGROUP *ngp, long lomark)
             LogName, MaxLength(start, start));
         return false;
     }
+    /* Check whether the high water mark is more than the low water mark
+     * minus one (in the case of an empty newsgroup).  If not, we consider
+     * that the newsgroup is empty and we increase the high water mark. */
+    l = atol(f2);
+    if (l+1 < lomark) {
+        syslog(L_NOTICE, RENUMBER, LogName, ngp->Name, "hi", l, lomark-1);
+        if (!FormatLong(f2, lomark-1, f3 - f2 - 1)) {
+            syslog(L_ERROR, NORENUMBER, LogName, ngp->Name, "hi");
+            return false;
+        }
+        ngp->Last = lomark-1;
+        ICDactivedirty++;
+    }
+    /* Update the low water mark. */
     l = atol(f3);
     if (lomark != l) {
         if (lomark < l)
