@@ -1,11 +1,10 @@
-/*  $Id: proc.c 8418 2009-04-12 21:08:22Z iulius $
-**
+/*
 **  Process control routines.
 */
 
 #include "config.h"
 #include "clibrary.h"
-#include "portable/wait.h"
+#include <sys/wait.h>
 
 #include "innd.h"
 
@@ -36,7 +35,7 @@ PROCreap(void)
 	    break;
 	}
 	for (pp = PROCtable, i = PROCtablesize; --i >= 0; pp++)
-	    if (pp->Pid == pid) {
+            if (pp->State == PSrunning && pp->Pid == pid) {
 		PROCneedscan = true;
 		pp->Status = WEXITSTATUS(status);
 		pp->State = PSdead;
@@ -47,18 +46,22 @@ PROCreap(void)
 }
 
 
+#ifdef HAVE_SIGACTION
+#define NO_SIGACTION_UNUSED UNUSED
+#else
+#define NO_SIGACTION_UNUSED
+#endif
+
 /*
 **  Signal handler that collects the processes, then resets the signal.
 */
 static void
-PROCcatchsignal(int s)
+PROCcatchsignal(int s NO_SIGACTION_UNUSED)
 {
     PROCreap();
 
 #ifndef HAVE_SIGACTION
     xsignal(s, PROCcatchsignal);
-#else
-    s = s;			/* ARGSUSED */
 #endif
 }
 
