@@ -1,5 +1,4 @@
-/*  $Id: python.c 10229 2018-01-28 21:21:21Z iulius $
-**
+/*
 **  Embed Python in the style of innd's Perl stuff.
 **
 **  Written by G.J. Andruk <meowing@banet.net> patterned after Tcl/Perl work
@@ -41,6 +40,18 @@
   typedef int Py_ssize_t;
 #endif
 
+#if PY_MAJOR_VERSION >= 3
+# define PyInt_FromLong PyLong_FromLong
+# define PyString_AS_STRING PyUnicode_AsUTF8
+# define PyString_FromStringAndSize PyBytes_FromStringAndSize
+# define PyString_InternFromString PyUnicode_InternFromString
+# define PYBUFF_FROMMEMORY(str, len) \
+      PyMemoryView_FromMemory((str), (len), PyBUF_WRITE)
+#else
+# define PYBUFF_FROMMEMORY(str, len) \
+      PyBuffer_FromMemory((str), (len))
+#endif
+
 #include "clibrary.h"
 
 #include "inn/innconf.h"
@@ -48,7 +59,6 @@
 #include "innd.h"
 
 bool		PythonFilterActive;
-char		*filterPath;	/* This gets set in art.c. */
 PyObject	*PYFilterObject = NULL;
 PyObject	*PYFilterModule = NULL;
 
@@ -56,7 +66,7 @@ PyObject	*PYFilterModule = NULL;
 PyObject	*PYheaders = NULL;
 PyObject	**PYheaditem;
 PyObject	**PYheadkey;
-PyObject	*PYpathkey, *PYlineskey, *PYbodykey;
+PyObject	*PYlineskey, *PYbodykey;
 
 /*  External functions. */
 PyObject	*msgid_method = NULL;
@@ -126,7 +136,7 @@ PYartfilter(const ARTDATA *data, char *artBody, long artLen, int lines)
     hdrnum = 0;
     for (i = 0 ; i < MAX_ARTHEADER ; i++) {
 	if (HDR_FOUND(i)) {
-	    PYheaditem[hdrnum] = PyBuffer_FromMemory(HDR(i), HDR_LEN(i));
+            PYheaditem[hdrnum] = PYBUFF_FROMMEMORY(HDR(i), HDR_LEN(i));
 	} else
 	    PYheaditem[hdrnum] = Py_None;
 	PyDict_SetItem(PYheaders, PYheadkey[hdrnum], PYheaditem[hdrnum]);
@@ -135,7 +145,7 @@ PYartfilter(const ARTDATA *data, char *artBody, long artLen, int lines)
 
     /* ...then the body... */
     if (artLen && artBody != NULL)
-        PYheaditem[hdrnum] = PyBuffer_FromMemory(artBody, --artLen);
+        PYheaditem[hdrnum] = PYBUFF_FROMMEMORY(artBody, --artLen);
     else
         PYheaditem[hdrnum] = Py_None;
     PyDict_SetItem(PYheaders, PYbodykey, PYheaditem[hdrnum++]);
@@ -198,7 +208,7 @@ PYmidfilter(char *messageID, int msglen)
 **  Tell the external module about innd's state.
 */
 void
-PYmode(OPERATINGMODE Mode, OPERATINGMODE NewMode, char *reason)
+PYmode(OPERATINGMODE CurrentMode, OPERATINGMODE NewMode, char *reason)
 {
     PyObject	*result;
     char	oldmode[10], newmode[10];
@@ -206,7 +216,7 @@ PYmode(OPERATINGMODE Mode, OPERATINGMODE NewMode, char *reason)
     if (!PythonFilterActive || PYFilterObject == NULL || mode_method == NULL)
 	return;
 
-    switch (Mode) {
+    switch (CurrentMode) {
     default:		strlcpy(oldmode, "unknown", 10);	break;
     case OMrunning:	strlcpy(oldmode, "running", 10);	break;
     case OMpaused:	strlcpy(oldmode, "paused", 10);		break;
@@ -304,7 +314,7 @@ PY_addhist(PyObject *self UNUSED, PyObject *args)
     char	*msgid;
     Py_ssize_t  msgidlen;
     char	*articlepaths = (char *) "";
-    char	tbuff[12];
+    char	tbuff[32];
     char	*parambuf[6];
 
     if (!PyArg_ParseTuple(args, (char *) "s#", &msgid, &msgidlen))
@@ -514,7 +524,8 @@ PY_hashstring(PyObject *self UNUSED, PyObject *args)
 	/* Compress out multiple whitespace in the trimmed string.  We
 	 * do a copy because this is probably an original art
 	 * buffer. */
-	workstring =  memcpy(xmalloc(worksize), wpos, worksize);
+        workstring = xmalloc(worksize + 1);
+        memcpy(workstring, wpos, worksize);
         wasspace = false;
 	p = wpos;
 	q = workstring;
@@ -564,6 +575,36 @@ static PyMethodDef INNPyMethods[] = {
     METHOD(NULL,              NULL,               0,            "")
 };
 
+#if PY_MAJOR_VERSION >= 3
+static struct PyModuleDef INNPyModule = {
+    PyModuleDef_HEAD_INIT,                /* m_base */
+    (char *) "INN",                       /* m_name */
+    (char *) "innd Python filter hook",   /* m_doc */
+    -1,                                   /* m_size */
+    INNPyMethods,                         /* m_methods */
+    NULL,                                 /* m_slots */
+    NULL,                                 /* m_traverse */
+    NULL,                                 /* m_clear */
+    NULL,                                 /* m_free */
+};
+
+PyMODINIT_FUNC
+PyInit_INN(void) {
+    PyObject *module = PyModule_Create(&INNPyModule);
+
+    if (module == NULL)
+        syslog(L_ERROR, "failed to create innd python module");
+
+    return module;
+}
+#else
+void
+PyInit_INN(void) {
+    if (Py_InitModule3((char *) "INN", INNPyMethods,
+                       (char *) "innd Python filter hook") == NULL)
+        syslog(L_ERROR, "failed to initialize innd python module");
+}
+#endif
 
 
 /*
@@ -642,8 +683,8 @@ PYreadfilter(void)
     PyObject	*result;
 
     if (!Py_IsInitialized()) {
-	syslog(L_ERROR, "python is not initialized");
-	return 0;
+        syslog(L_NOTICE, "python is not initialized");
+        return 0;
     }
 
     /* If there is a filter running, let it clean up first. */
@@ -698,12 +739,25 @@ PYsetup(void)
 {
     const ARTHEADER *hp;
     size_t hdrcount;
+    char *path;
 
-    /* Add path for nnrpd module.  The environment variable PYTHONPATH
+    path = concatpath(innconf->pathfilter, INN_PATH_PYTHON_STARTUP);
+    if (access(path, R_OK) < 0) {
+        syslog(L_NOTICE, "pyfilter %s not installed", path);
+        PYfilter(false);
+        free(path);
+        return;
+    }
+    free(path);
+
+    /* Add path for innd module.  The environment variable PYTHONPATH
      * does it; one can also append innconf->pathfilter to sys.path once
      * Python has been initialized. */
     setenv("PYTHONPATH", innconf->pathfilter, 1);
 
+    /* Build a module interface to certain INN functions. */
+    PyImport_AppendInittab("INN", &PyInit_INN);
+
     /* Load up the interpreter ;-O */
     Py_Initialize();
 
@@ -718,12 +772,10 @@ PYsetup(void)
 	return;
     }
 
-    /* Build a module interface to certain INN functions. */
-    Py_InitModule((char *) "INN", INNPyMethods);
-
     PYFilterModule = PyImport_ImportModule((char *) INN_PATH_PYTHON_STARTUP_M);
     if (PYFilterModule == NULL)
-	syslog(L_ERROR, "failed to import external python module");
+	syslog(L_ERROR, "failed to import external %s python module",
+           INN_PATH_PYTHON_STARTUP_M);
 
     if (PYFilterObject == NULL) {
 	syslog(L_ERROR, "python filter object is not defined");
@@ -742,10 +794,9 @@ PYsetup(void)
     PYheaditem = xmalloc((hdrcount + 2) * sizeof(PyObject *));
     PYheadkey = xmalloc(hdrcount * sizeof(PyObject *));
 
-    /* Preallocate keys for the article dictionary */
+    /* Preallocate keys for the article dictionary. */
     for (hp = ARTheaders; hp < ARRAY_END(ARTheaders); hp++)
 	PYheadkey[hp - ARTheaders] = PyString_InternFromString(hp->Name);
-    PYpathkey = PyString_InternFromString("Path");
     PYlineskey = PyString_InternFromString("__LINES__");
     PYbodykey = PyString_InternFromString("__BODY__");
 
