@@ -1,5 +1,4 @@
-/*  $Id: rc.c 9024 2010-03-21 16:49:30Z iulius $
-**
+/*
 **  Routines for the remote connect channel.  Create an Internet stream
 **  socket that processes connect to.  If the incoming site is not one of
 **  our feeds, then we optionally pass the connection off to the standard
@@ -11,8 +10,10 @@
 #include <errno.h>
 #include <netdb.h>
 
+#include "inn/fdflag.h"
 #include "inn/innconf.h"
 #include "inn/network.h"
+#include "inn/network-innbind.h"
 #include "inn/vector.h"
 #include "innd.h"
 
@@ -32,7 +33,7 @@
 typedef struct _REMOTEHOST {
     char	*Label;         /* Peer label */
     char	*Name;          /* Hostname */
-    struct sockaddr_storage Address;     /* List of ip adresses */
+    struct sockaddr_storage Address;     /* List of ip addresses */
     char	*Password;      /* Optional password */
     char 	*Identd;	/* Optional identd */
     bool	Streaming;      /* Streaming allowed ? */
@@ -64,7 +65,7 @@ static char		*RCslaveflag;
 static char		*RCnnrpd = NULL;
 static char		*RCnntpd = NULL;
 static CHANNEL		**RCchan;
-static int		chanlimit;
+static unsigned int	chanlimit;
 static REMOTEHOST_DATA	*RCpeerlistfile;
 static REMOTEHOST	*RCpeerlist;
 static int		RCnpeerlist;
@@ -126,7 +127,7 @@ GoodIdent(int fd, char *identd)
     struct sockaddr *s_local;
     struct sockaddr *s_distant = NULL;
     int ident_fd;
-    socklen_t len;
+    socklen_t local_len, distant_len;
     int port1,port2;
     ssize_t lu;
     char buf[80], *buf2;
@@ -135,33 +136,33 @@ GoodIdent(int fd, char *identd)
          return true;
 
     s_local = xmalloc(sizeof(struct sockaddr_storage));
-    len = sizeof(struct sockaddr_storage);
-    if ((getsockname(fd, s_local, &len)) < 0) {
+    local_len = sizeof(struct sockaddr_storage);
+    if ((getsockname(fd, s_local, &local_len)) < 0) {
 	syslog(L_ERROR, "can't do getsockname for identd");
         goto fail;
     }
     s_distant = xmalloc(sizeof(struct sockaddr_storage));
-    len = sizeof(struct sockaddr_storage);
-    if ((getpeername(fd, s_distant, &len)) < 0) {
+    distant_len = sizeof(struct sockaddr_storage);
+    if ((getpeername(fd, s_distant, &distant_len)) < 0) {
 	syslog(L_ERROR, "can't do getpeername for identd");
         goto fail;
     }
 #ifdef HAVE_INET6
     if (s_local->sa_family == AF_INET6)
     {
-	port1 = ntohs(((struct sockaddr_in6 *) s_local)->sin6_port);
-	port2 = ntohs(((struct sockaddr_in6 *) s_distant)->sin6_port);
-	((struct sockaddr_in6 *) s_local)->sin6_port = 0;
-	((struct sockaddr_in6 *) s_distant)->sin6_port = htons(PORT_IDENTD);
+        port1 = ntohs(((struct sockaddr_in6 *) (void *) s_local)->sin6_port);
+        port2 = ntohs(((struct sockaddr_in6 *) (void *) s_distant)->sin6_port);
+        ((struct sockaddr_in6 *) (void *) s_local)->sin6_port = 0;
+        ((struct sockaddr_in6 *) (void *) s_distant)->sin6_port = htons(PORT_IDENTD);
 	ident_fd = socket(PF_INET6, SOCK_STREAM, 0);
     } else
 #endif
     if (s_local->sa_family == AF_INET)
     {
-	port1 = ntohs(((struct sockaddr_in *) s_local)->sin_port);
-        port2 = ntohs(((struct sockaddr_in *) s_distant)->sin_port);
-	((struct sockaddr_in *) s_local)->sin_port = 0;
-	((struct sockaddr_in *) s_distant)->sin_port = htons(PORT_IDENTD);
+        port1 = ntohs(((struct sockaddr_in *) (void *) s_local)->sin_port);
+        port2 = ntohs(((struct sockaddr_in *) (void *) s_distant)->sin_port);
+        ((struct sockaddr_in *) (void *) s_local)->sin_port = 0;
+        ((struct sockaddr_in *) (void *) s_distant)->sin_port = htons(PORT_IDENTD);
 	ident_fd = socket(PF_INET, SOCK_STREAM, 0);
     } else
     {
@@ -172,12 +173,12 @@ GoodIdent(int fd, char *identd)
 	syslog(L_ERROR, "can't open socket for identd (%m)");
         goto fail;
     }
-    if (bind(ident_fd, s_local, SA_LEN(s_local)) < 0) {
+    if (bind(ident_fd, s_local, local_len) < 0) {
 	syslog(L_ERROR, "can't bind socket for identd (%m)");
         close(ident_fd);
         goto fail;
     }
-    if (connect(ident_fd, s_distant, SA_LEN(s_distant)) < 0) {
+    if (connect(ident_fd, s_distant, distant_len) < 0) {
 	syslog(L_ERROR, "can't connect to identd (%m)");
         close(ident_fd);
         goto fail;
@@ -381,7 +382,7 @@ RChandoff(int fd, HANDOFF h)
         syslog(L_ERROR, "fd %d cant setsockopt(KEEPALIVE) %m", fd);
 #endif /* defined(SOL_SOCKET) && defined(SO_KEEPALIVE) */
 
-    if (nonblocking(fd, false) < 0)
+    if (!fdflag_nonblocking(fd, false))
 	syslog(L_ERROR, "%s cant nonblock %d in RChandoff %m", LogName, fd);
     switch (h) {
     default:
@@ -425,6 +426,7 @@ RCreader(CHANNEL *cp)
     struct sockaddr_storage	remote;
     socklen_t		size;
     int                 i;
+    unsigned int        j;
     REMOTEHOST          *rp;
     CHANNEL		*new;
     char		*name;
@@ -437,12 +439,12 @@ RCreader(CHANNEL *cp)
     char		buff[SMBUF];
     char                addr[INET6_ADDRSTRLEN];
 
-    for (i = 0 ; i < chanlimit ; i++) {
-	if (RCchan[i] == cp) {
+    for (j = 0 ; j < chanlimit ; j++) {
+	if (RCchan[j] == cp) {
 	    break;
 	}
     }
-    if (i == chanlimit) {
+    if (j == chanlimit) {
 	syslog(L_ERROR, "%s internal RCreader wrong channel 0x%p",
 		LogName, (void *)cp);
 	return;
@@ -455,11 +457,6 @@ RCreader(CHANNEL *cp)
 	    syslog(L_ERROR, "%s cant accept RCreader %m", LogName);
 	return;
     }
-    if (!network_kill_options(fd, (struct sockaddr *) &remote)) {
-        if (close(fd) < 0)
-            syslog(L_ERROR, "%s cant close %d %m", LogName, fd);
-        return;
-    }
 
     /* If RemoteTimer is not zero, then check the limits on incoming
        connections on a total and per host basis.
@@ -661,7 +658,6 @@ RCreaddata(int *num, FILE *F, bool *toolong)
   char	*s;
   char	*t;
   char	*word;
-  bool	flag;
 
   *toolong = false;
   if (*RCbuff == '\0') {
@@ -673,7 +669,6 @@ RCreaddata(int *num, FILE *F, bool *toolong)
       return (NULL); /* Line too long */
     }
   }
-  p = RCbuff;
   do {
      /* Ignore blank and comment lines. */
      if ((p = strchr(RCbuff, '\n')) != NULL)
@@ -683,24 +678,22 @@ RCreaddata(int *num, FILE *F, bool *toolong)
 	   *p = '\0';
      }
      for (p = RCbuff; *p == ' ' || *p == '\t' ; p++);
-     flag = true;
      if (*p == '\0' && !feof (F)) {
-       flag = false;
        fgets(RCbuff, sizeof RCbuff, F);
        (*num)++;
        if (strlen (RCbuff) == sizeof RCbuff) {
-	 *toolong = true;
-	 return (NULL); /* Line too long */
+           *toolong = true;
+           return (NULL); /* Line too long */
        }
        continue;
      }
      break;
-  } while (!feof (F) || !flag);
+  } while (!feof (F));
 
   if (*p == '"') { /* double quoted string ? */
     p++;
     do {
-      for (t = p; (*t != '"' || (*t == '"' && *(t - 1) == '\\')) &&
+      for (t = p; (*t != '"' || *(t - 1) == '\\') &&
 	     *t != '\0'; t++);
       if (*t == '\0') {
 	*t++ = '\n';
@@ -768,7 +761,6 @@ RCreadfile (REMOTEHOST_DATA **data, REMOTEHOST **list, int *count,
     int			infocount;
     int                 groupcount;
     int                 maxgroup;
-    REMOTEHOST_DATA 	*dt;
     REMOTEHOST		*rp;
     char		*word;
     REMOTEHOST		*groups;
@@ -815,7 +807,8 @@ RCreadfile (REMOTEHOST_DATA **data, REMOTEHOST **list, int *count,
 	syslog(L_FATAL, "%s cant read %s: %m", LogName, filename);
 	exit(1);
     }
-    dt = *data = xmalloc(sizeof(REMOTEHOST_DATA));
+
+    *data = xmalloc(sizeof(REMOTEHOST_DATA));
     rp = *list = xmalloc(sizeof(REMOTEHOST));
 
 #if	!defined(HAVE_UNIX_DOMAIN_SOCKETS)
@@ -1519,8 +1512,6 @@ RCwritelist(char *filename)
         if (*p == '/')
 	   q = p + 1;
 
-    fprintf (F, "##  $Id: rc.c 9024 2010-03-21 16:49:30Z iulius $\n");
-    fprintf (F, "##\n");
     fprintf (F, "##  %s -- Configuration of incoming news feeds\n", q);
     free(r);
     fprintf (F, "##\n\n");
@@ -1736,28 +1727,90 @@ void
 RCsetup(void)
 {
     CHANNEL *rc;
-    int count, i, start;
+    unsigned int count, i, start;
     int *fds;
     bool okay;
+    const char *s;
+
+    /* If systemd socket activation is used, backup its environment variables,
+       which are unset by sd_listen_fds(), to be able to restore them later in
+       CCxexec() before starting a new innd process. */
+    s = getenv("LISTEN_FDS");
+    if (s != NULL) {
+        if (setenv("INN_BACKUP_LISTEN_FDS", s, true) != 0)
+            die("setenv failed for INN_BACKUP_LISTEN_FDS");
+    }
+    s = getenv("LISTEN_PID");
+    if (s != NULL) {
+        if (setenv("INN_BACKUP_LISTEN_PID", s, true) != 0)
+            die("setenv failed for INN_BACKUP_LISTEN_PID");
+    }
+
+    /* Ignore any configured ports if socket activation is being used. */
+    count = sd_listen_fds(true);
+    if (count > 0) {
+        char *p;
+
+        xasprintf(&p, "%u", count);
+        if (setenv("INN_SD_LISTEN_FDS_COUNT", p, true) != 0)
+            die("setenv failed for INN_SD_LISTEN_FDS_COUNT");
+
+        free(p);
+
+        /* Make sure that the RCchan array is of the appropriate size. */
+        if (chanlimit == 0) {
+            chanlimit = count;
+            RCchan = xmalloc(chanlimit * sizeof(CHANNEL *));
+            start = 0;
+        } else {
+            for (start = 0; start < chanlimit; start++) {
+                if (RCchan[start] == NULL)
+                    break;
+                 if (chanlimit - start < count) {
+                     chanlimit += count - (chanlimit - start);
+                     RCchan = xrealloc(RCchan, chanlimit * sizeof(CHANNEL *));
+                 }
+            }
+        }
+
+        for (i = SD_LISTEN_FDS_START; i < SD_LISTEN_FDS_START + count; i++) {
+            if (sd_is_socket(i, AF_UNSPEC, SOCK_STREAM, 1) <= 0)
+                die("SERVER can't use socket-activated non-AF_INET socket %u",
+                    i);
+            rc = CHANcreate(i, CTremconn, CSwaiting, RCreader, RCwritedone);
+            notice("%s rcsetup %s", LogName, CHANname(rc));
+            RCHANadd(rc);
+            RCchan[start + i - SD_LISTEN_FDS_START] = rc;
+        }
+
+        /* Get the list of hosts we handle. */
+        RCreadlist();
+
+        return;
+    }
 
     /* If neither bindaddress or bindaddress6 are set, we have to do this the
        hard way.  Either way, allocate an fds array to hold the file
        descriptors.  Two is probably always sufficient, but we can't tell for
        sure how many will be returned by getaddrinfo and this only happens on
        startup so doing a bit of memory allocation won't hurt. */
-    if (innconf->bindaddress == NULL && innconf->bindaddress6 == NULL)
-        network_bind_all(innconf->port, &fds, &count);
-    else {
+    if (innconf->bindaddress == NULL && innconf->bindaddress6 == NULL) {
+        network_innbind_all(SOCK_STREAM, innconf->port, &fds, &count);
+    } else {
         if (innconf->bindaddress != NULL && innconf->bindaddress6 != NULL)
             count = 2;
         else
             count = 1;
         fds = xmalloc(count * sizeof(int));
         i = 0;
-        if (innconf->bindaddress6 != NULL)
-            fds[i++] = network_bind_ipv6(innconf->bindaddress6, innconf->port);
-        if (innconf->bindaddress != NULL)
-            fds[i] = network_bind_ipv4(innconf->bindaddress, innconf->port);
+        if (innconf->bindaddress6 != NULL) {
+            fds[i++] = network_innbind_ipv6(SOCK_STREAM, innconf->bindaddress6,
+                                            innconf->port);
+        }
+        if (innconf->bindaddress != NULL) {
+            fds[i] = network_innbind_ipv4(SOCK_STREAM, innconf->bindaddress,
+                                          innconf->port);
+        }
     }
 
     /* Make sure that the RCchan array is of the appropriate size. */
@@ -1783,7 +1836,7 @@ RCsetup(void)
     for (i = 0; i < count; i++) {
         if (fds[i] < 0)
             continue;
-        if (listen(fds[i], MAXLISTEN) < 0) {
+        if (listen(fds[i], innconf->maxlisten) < 0) {
             if (i != 0 && errno == EADDRINUSE)
                 continue;
             syswarn("SERVER cant listen to socket");
@@ -1801,6 +1854,8 @@ RCsetup(void)
 
     /* Get the list of hosts we handle. */
     RCreadlist();
+
+    free(fds);
 }
 
 
@@ -1810,12 +1865,18 @@ RCsetup(void)
 void
 RCclose(void)
 {
-    REMOTEHOST	*rp;
-    int		i;
+    REMOTEHOST   *rp;
+    int          i;
+    unsigned int j;
 
-    for (i = 0 ; i < chanlimit ; i++) {
-	if (RCchan[i] != NULL) {
-	    CHANclose(RCchan[i], CHANname(RCchan[i]));
+    for (j = 0 ; j < chanlimit ; j++) {
+        if (RCchan[j] != NULL) {
+            /* Do not close the listening sockets if socket activation is being
+               used because they need to be passed as-is by CCxexec() to the
+               next innd process. */
+            if (CHANsystemdsa(RCchan[j]))
+                continue;
+            CHANclose(RCchan[j], CHANname(RCchan[j]));
 	} else {
 	    break;
 	}
