@@ -1,5 +1,4 @@
-/*  $Id: site.c 7747 2008-04-06 13:16:45Z iulius $
-**
+/*
 **  Routines to implement site-feeding.  Mainly working with channels to
 **  do buffering and determine what to send.
 */
@@ -7,6 +6,7 @@
 #include "config.h"
 #include "clibrary.h"
 
+#include "inn/fdflag.h"
 #include "inn/innconf.h"
 #include "innd.h"
 
@@ -333,7 +333,7 @@ SITEwritefromflags(SITE *sp, ARTDATA *Data)
     HDRCONTENT		*hc = Data->HdrContent;
     static char		ITEMSEP[] = " ";
     static char		NL[] = "\n";
-    char		pbuff[12];
+    char		pbuff[32];
     char	        *p;
     bool	        Dirty;
     struct buffer       *bp;
@@ -446,11 +446,11 @@ SITEwritefromflags(SITE *sp, ARTDATA *Data)
 	    buffer_append(bp, HDR(HDR__MESSAGE_ID), HDR_LEN(HDR__MESSAGE_ID));
 	    break;
 	case FEED_FNLNAMES:
-	    if (sp->FNLnames.data) {
+	    if (sp->FNLnames.left != 0) {
 		/* Funnel; write names of our sites that got it. */
 		if (Dirty)
 		    buffer_append(bp, ITEMSEP, strlen(ITEMSEP));
-		buffer_append(bp, sp->FNLnames.data, sp->FNLnames.used);
+		buffer_append(bp, sp->FNLnames.data, sp->FNLnames.left);
 	    }
 	    else {
 		/* Not funnel; write names of all sites that got it. */
@@ -515,25 +515,23 @@ SITEsend(SITE *sp, ARTDATA *Data)
     case FTprogram:
 	/* Set up the argument vector. */
 	if (sp->FNLwantsnames) {
-	    i = strlen(sp->Param) + sp->FNLnames.used;
+	    i = strlen(sp->Param) + sp->FNLnames.left;
 	    if (i + (sizeof(TOKEN) * 2) + 3 >= sizeof buff) {
 		syslog(L_ERROR, "%s toolong need %lu for %s",
 		    sp->Name, (unsigned long) (i + (sizeof(TOKEN) * 2) + 3),
 		    sp->Name);
 		break;
 	    }
-	    temp = xmalloc(i + 1);
 	    p = strchr(sp->Param, '*');
 	    *p = '\0';
-	    strlcpy(temp, sp->Param, i + 1);
-	    strlcat(temp, sp->FNLnames.data, i + 1);
-	    strlcat(temp, &p[1], i + 1);
+	    xasprintf(&temp, "%s%.*s%s", sp->Param, (int) sp->FNLnames.left,
+		      sp->FNLnames.data, &p[1]);
 	    *p = '*';
 	    snprintf(buff, sizeof(buff), temp, Data->TokenText);
 	    free(temp);
-	}
-	else
+	} else {
 	    snprintf(buff, sizeof(buff), sp->Param, Data->TokenText);
+        }
 
 	if (NeedShell(buff, (const char **)argv, (const char **)ARRAY_END(argv))) {
 	    argv[0] = SITEshell;
@@ -601,7 +599,7 @@ SITEstartprocess(SITE *sp)
 	return false;
     }
 #endif
-    close_on_exec(pan[PIPE_WRITE], true);
+    fdflag_close_exec(pan[PIPE_WRITE], true);
 
     /* Set up the argument vector. */
     process = xstrdup(sp->Param);
@@ -1066,6 +1064,8 @@ SITEfree(SITE *sp)
 	free(sp->FNLnames.data);
 	sp->FNLnames.data = NULL;
 	sp->FNLnames.size = 0;
+	sp->FNLnames.left = 0;
+	sp->FNLnames.used = 0;
     }
     if (sp->HashFeedList) {
         for (hf = sp->HashFeedList; hf; hf = hn) {
@@ -1145,87 +1145,87 @@ SITEinfo(struct buffer *bp, SITE *sp, const bool Verbose)
 	return;
     }
 
-    buffer_sprintf(bp, true, "%s%s:\t", sp->Name, sp->IsMaster ? "(*)" : "");
+    buffer_append_sprintf(bp, "%s%s:\t", sp->Name, sp->IsMaster ? "(*)" : "");
 
     if (sp->Type == FTfunnel) {
 	sp = &Sites[sp->Funnel];
-        buffer_sprintf(bp, true, "funnel -> %s: ", sp->Name);
+        buffer_append_sprintf(bp, "funnel -> %s: ", sp->Name);
     }
 
     switch (sp->Type) {
     default:
-        buffer_sprintf(bp, true, "unknown feed type %d", sp->Type);
+        buffer_append_sprintf(bp, "unknown feed type %d", sp->Type);
 	break;
     case FTerror:
     case FTfile:
-        buffer_sprintf(bp, true, "file");
+        buffer_append_sprintf(bp, "file");
 	if (sp->Buffered)
-            buffer_sprintf(bp, true, " buffered(%lu)",
+            buffer_append_sprintf(bp, " buffered(%lu)",
                            (unsigned long) sp->Buffer.left);
 	else if ((cp = sp->Channel) == NULL)
-            buffer_sprintf(bp, true, " no channel?");
+            buffer_append_sprintf(bp, " no channel?");
 	else
-            buffer_sprintf(bp, true, " open fd=%d, in mem %lu", cp->fd,
+            buffer_append_sprintf(bp, " open fd=%d, in mem %lu", cp->fd,
                            (unsigned long) cp->Out.left);
 	break;
     case FTchannel:
-        buffer_sprintf(bp, true, "channel");
+        buffer_append_sprintf(bp, "channel");
 	goto common;
     case FTexploder:
-        buffer_sprintf(bp, true, "exploder");
+        buffer_append_sprintf(bp, "exploder");
 common:
 	if (sp->Process >= 0)
-            buffer_sprintf(bp, true, " pid=%ld", (long) sp->pid);
+            buffer_append_sprintf(bp, " pid=%ld", (long) sp->pid);
 	if (sp->Spooling)
-            buffer_sprintf(bp, true, " spooling");
+            buffer_append_sprintf(bp, " spooling");
         cp = sp->Channel;
 	if (cp == NULL)
-            buffer_sprintf(bp, true, " no channel?");
+            buffer_append_sprintf(bp, " no channel?");
 	else
-            buffer_sprintf(bp, true, " fd=%d, in mem %lu", cp->fd,
+            buffer_append_sprintf(bp, " fd=%d, in mem %lu", cp->fd,
                            (unsigned long) cp->Out.left);
 	break;
     case FTfunnel:
-        buffer_sprintf(bp, true, "recursive funnel");
+        buffer_append_sprintf(bp, "recursive funnel");
 	break;
     case FTlogonly:
-        buffer_sprintf(bp, true, "log only");
+        buffer_append_sprintf(bp, "log only");
 	break;
     case FTprogram:
-        buffer_sprintf(bp, true, "program");
+        buffer_append_sprintf(bp, "program");
 	if (sp->FNLwantsnames)
-            buffer_sprintf(bp, true, " with names");
+            buffer_append_sprintf(bp, " with names");
 	break;
     }
     buffer_append(bp, "\n", 1);
     if (Verbose) {
 	sep = "\t";
 	if (sp->Buffered && sp->Flushpoint) {
-            buffer_sprintf(bp, true, "%sFlush @ %lu", sep,
+            buffer_append_sprintf(bp, "%sFlush @ %lu", sep,
                            (unsigned long) sp->Flushpoint);
 	    sep = "; ";
 	}
 	if (sp->StartWriting || sp->StopWriting) {
-            buffer_sprintf(bp, true, "%sWrite [%ld..%ld]", sep,
+            buffer_append_sprintf(bp, "%sWrite [%ld..%ld]", sep,
                            sp->StopWriting, sp->StartWriting);
 	    sep = "; ";
 	}
 	if (sp->StartSpooling) {
-            buffer_sprintf(bp, true, "%sSpool @ %ld", sep, sp->StartSpooling);
+            buffer_append_sprintf(bp, "%sSpool @ %ld", sep, sp->StartSpooling);
 	    sep = "; ";
 	}
 	if (sep[0] != '\t')
             buffer_append(bp, "\n", 1);
 	if (sp->Spooling && sp->SpoolName)
-            buffer_sprintf(bp, true, "\tSpooling to \"%s\"\n", sp->SpoolName);
+            buffer_append_sprintf(bp, "\tSpooling to \"%s\"\n", sp->SpoolName);
         cp = sp->Channel;
 	if (cp != NULL) {
-            buffer_sprintf(bp, true, "\tChannel created %.12s",
+            buffer_append_sprintf(bp, "\tChannel created %.12s",
                            ctime(&cp->Started) + 4);
-            buffer_sprintf(bp, true, ", last active %.12s\n",
+            buffer_append_sprintf(bp, ", last active %.12s\n",
                            ctime(&cp->LastActive) + 4);
 	    if (cp->Waketime > Now.tv_sec)
-                buffer_sprintf(bp, true, "\tSleeping until %.12s\n",
+                buffer_append_sprintf(bp, "\tSleeping until %.12s\n",
                                ctime(&cp->Waketime) + 4);
 	}
 
