@@ -1,5 +1,4 @@
-/*  $Id: main.c 8781 2009-11-15 09:24:59Z iulius $
-**
+/*
 **  Main routines for the innfeed program.
 **
 **  Written by James Brister <brister@vix.com>
@@ -9,7 +8,6 @@
 #include "config.h"
 #include "clibrary.h"
 #include "portable/socket.h"
-#include "portable/time.h"
 
 #include <assert.h>
 #include <ctype.h>
@@ -21,8 +19,13 @@
 #include <sys/stat.h>
 #include <syslog.h>
 
+#ifdef HAVE_SYS_TIME_H
+# include <sys/time.h>
+#endif
+#include <time.h>
+
 #if defined(HAVE_UNIX_DOMAIN_SOCKETS)
-# include <sys/un.h>
+# include "portable/socket-unix.h"
 #endif
 
 #include "inn/innconf.h"
@@ -52,7 +55,9 @@ const char *InputFile ;
 char *configFile = NULL ;
 bool RollInputFile = false ;
 char *pidFile = NULL ;
-bool useMMap = false ;
+bool useMMap = true ;  /* Even if !HAVE_MMAP.  Only used when innfeed is given
+                        * file names to send instead of storage API tokens,
+                        * which is a fairly rare use case. */
 void (*gPrintInfo) (void) ;
 char *dflTapeDir;
 /* these are used by imapfeed */
@@ -69,6 +74,9 @@ extern char *optarg ;           /* needed for Solaris */
 extern int optind;
 #endif
 extern bool genHtml ;
+extern bool debugShrinking;
+extern bool fastExit;
+extern unsigned int stdioFdMax;
 
 extern void openInputFile (void);
 
@@ -84,7 +92,7 @@ static void sigquit (int sig) ;
 static void sighup (int sig) ;
 static void sigterm (int sig) ;
 static void sigusr (int sig) ;
-static void usage (int) ;
+static void usage (int) __attribute__ ((__noreturn__)) ;
 static void gprintinfo (void) ;
 static void openLogFile (void) ;
 static void writePidFile (void) ;
@@ -136,12 +144,6 @@ int main (int argc, char **argv)
       exit(1);
   }
 
-#if defined (HAVE_MMAP)
-  useMMap = true ;
-#else
-  useMMap = false ;
-#endif
-
   message_handlers_die (2, error_log_stderr_date, message_log_syslog_err) ;
   message_handlers_warn (1, message_log_syslog_warning);
   message_handlers_notice (1, message_log_syslog_notice) ;
@@ -289,7 +291,7 @@ int main (int argc, char **argv)
 
   if ( !checkConfig ) 
     {
-      notice ("ME starting at %s\n (%s)", dateString, INN_VERSION_STRING);
+      notice("ME starting at %s (%s)", dateString, INN_VERSION_STRING);
     }
 
   val = true;
@@ -298,8 +300,8 @@ int main (int argc, char **argv)
       exit(1);
   }
   if (!SMinit()) {
-      d_printf(0, "Storage manager initialization failed\n");
-      syslog(LOG_ERR, "Storage manager initialization failed\n");
+      d_printf(0, "Storage manager initialization failed -- it is OK after a change in storage methods\n");
+      syslog(LOG_ERR, "Storage manager initialization failed -- it is OK after a change in storage methods\n");
       exit(1);
   }
 
@@ -363,6 +365,7 @@ int main (int argc, char **argv)
   debugFile = concatpath(innconf->pathlog, DEBUG_FILE);
   if (loggingLevel == 0 && fileExistsP (debugFile))
     loggingLevel = 1 ;
+  free(debugFile);
 
   if (logFile == NULL && ! isatty (fileno (stderr)))
     logFile = concatpath(innconf->pathlog, LOG_FILE);
@@ -491,54 +494,56 @@ static void usage (int val)
   fprintf (stderr,"\t-a dir      Use the given directory as the top of the article spool\n") ;
 
   fprintf (stderr,"\t-b dir      Use the given directory as the the storage\n");
-  fprintf (stderr,"\t            place for backlog files and lock files.\n");
+  fprintf (stderr,"\t            place for backlog files and lock files\n");
 
   fprintf (stderr,"\t-c file     Use the given file as the config file instead of the\n");
   fprintf (stderr,"\t            default of %s\n",CONFIG_FILE);
 
-  fprintf (stderr,"\t-C          Check the config file and then exit.\n") ;
-  fprintf (stderr,"\t-d num      set the logging level to num (an integer).\n");
-  fprintf (stderr,"\t            Larger value means more logging. 0 means no\n");
-  fprintf (stderr,"\t            logging. The default is 0\n");
+  fprintf (stderr,"\t-C          Check the config file and then exit\n") ;
+  fprintf (stderr,"\t-d num      Set the logging level to num (an integer).\n");
+  fprintf (stderr,"\t            Larger value means more logging.  0 means no\n");
+  fprintf (stderr,"\t            logging.  The default is 0\n");
 
   fprintf (stderr,"\t-e bytes    Keep the output backlog files to no bigger\n");
   fprintf (stderr,"\t            than %.2f times this number\n",LIMIT_FUDGE);
 
-  fprintf (stderr,"\t-h          print this message\n");
+  fprintf (stderr,"\t-h          Print this message\n");
 
-  fprintf (stderr,"\t-l file     redirect stderr and stdout to the given file.\n");
-  fprintf (stderr,"\t            When run under INN they normally are redirected to\n");
-  fprintf (stderr,"\t            /dev/null. This is needed if using '-d'.\n");
+  fprintf (stderr,"\t-l file     Redirect stderr and stdout to the given file.\n");
+  fprintf (stderr,"\t            When run under INN, they normally are redirected to\n");
+  fprintf (stderr,"\t            /dev/null.  This is needed if using '-d'\n");
 
   fprintf (stderr,"\t-m          Log information on all missing articles\n");
 
   fprintf (stderr,"\t-M          Turn *off* use of mmap\n") ;
 #if ! defined (HAVE_MMAP)
-  fprintf (stderr,"\t            (a no-op as this excutable has been built without mmap support\n") ;
+  fprintf (stderr,"\t            (a no-op as this excutable has been built without mmap support)\n") ;
 #endif
+  fprintf (stderr,"\t-o bytes    Set a limit for the maximum number of bytes of article\n");
+  fprintf (stderr,"\t            data innfeed is supposed to keep in memory\n");
 
   fprintf (stderr,"\t-p file     Write the process id to the given file\n") ;
-  fprintf (stderr,"\t            instead of the default of %s\n",PID_FILE);
-  fprintf (stderr,"\t            A relative path is relative to %s\n", innconf->pathrun) ;
+  fprintf (stderr,"\t            instead of the default of %s;\n",PID_FILE);
+  fprintf (stderr,"\t            a relative path is relative to %s\n", innconf->pathrun) ;
 
-  fprintf (stderr,"\t-s command  run the given command in a subprocess and use\n");
+  fprintf (stderr,"\t-s command  Run the given command in a subprocess and use\n");
   fprintf (stderr,"\t            its output as article information instead of\n");
   fprintf (stderr,"\t            running under innd\n");
 
-  fprintf (stderr,"\t-S file     Use the give filename instead of innfeed.status\n") ;
-  fprintf (stderr,"\t            relative pathnames start from %s\n", innconf->pathlog) ;
+  fprintf (stderr,"\t-S file     Use the given filename instead of innfeed.status;\n") ;
+  fprintf (stderr,"\t            relative path names start from %s\n", innconf->pathlog) ;
 
-  fprintf (stderr,"\t-v          print version information\n");
+  fprintf (stderr,"\t-v          Print version information\n");
 
   fprintf (stderr,"\t-x          Do not read any article information off stdin,\n");
   fprintf (stderr,"\t            but simply process backlog files and then exit\n");
   fprintf (stderr,"\t            when done\n");
 
-  fprintf (stderr,"\t-y          Add peers dynamically. If an unrecognized peername\n");
+  fprintf (stderr,"\t-y          Add peers dynamically.  If an unrecognized peer name\n");
   fprintf (stderr,"\t            is received from innd, then it is presumed to also\n");
-  fprintf (stderr,"\t            be the ip name and a new peer binding is set up\n");
+  fprintf (stderr,"\t            be the IP name and a new peer binding is set up\n");
 
-  fprintf (stderr,"\t-z          have each of the connections issue their own stats\n");
+  fprintf (stderr,"\t-z          Have each of the connections issue their own stats\n");
   fprintf (stderr,"\t            whenever they close, or whenever their controller\n");
   fprintf (stderr,"\t            issues its own stats\n");
 
@@ -571,6 +576,9 @@ static void sighup (int sig UNUSED)
     }
 
   configHosts (talkToSelf) ;
+
+  notice ("ME reloading log file %s", logFile) ;
+  openLogFile() ;
 }
 
 static void sigemt (int sig UNUSED)
@@ -955,7 +963,7 @@ static void mainCleanup (void)
 void mainLogStatus (FILE *fp)
 {
   fprintf (fp,"%sGlobal configuration parameters:%s\n",
-           genHtml ? "<B>" : "", genHtml ? "</B>" : "") ;
+           genHtml ? "<strong>" : "", genHtml ? "</strong>" : "") ;
   fprintf (fp,"          Mode: ") ;
   if (InputFile != NULL)
     fprintf (fp,"Funnel file") ;
@@ -969,7 +977,10 @@ void mainLogStatus (FILE *fp)
   fprintf (fp,"    News spool: %s\n",newsspool) ;
   fprintf (fp,"      Pid file: %s\n",pidFile) ;
   fprintf (fp,"      Log file: %s\n",(logFile == NULL ? "(none)" : logFile));
-  fprintf (fp,"   Debug level: %2ld                Mmap: %s\n",
-           (long)loggingLevel,boolToString(useMMap)) ;
+  fprintf (fp,"   Debug level: %-5u        Debug shrinking: %s\n",
+           loggingLevel, boolToString(debugShrinking));
+  fprintf (fp,"     Fast exit: %-5s            stdio-fdmax: %u\n",
+           boolToString(fastExit), stdioFdMax);
+  fprintf (fp,"          Mmap: %s\n", boolToString(useMMap));
   fprintf (fp,"\n") ;
 }
