@@ -1,5 +1,4 @@
-/*  $Id: misc.h 9275 2011-07-17 19:09:26Z iulius $
-**
+/*
 **  Miscellaneous utility functions for innfeed.
 **
 **  Written by James Brister <brister@vix.com>
@@ -9,6 +8,7 @@
 #define misc_h__
 
 #include "config.h"
+#include "portable/macros.h"
 
 #include <stdarg.h>
 #include <sys/types.h>
@@ -55,7 +55,7 @@ void d_printf (unsigned int level, const char *fmt, ...)
 /* for the gethostbyname() error code */
 const char *host_err_str (void) ;
 
-/* parse a reponse line into it's code and body. *rest will end up pointing
+/* parse a response line into it's code and body. *rest will end up pointing
    into the middle of p */
 bool getNntpResponse (char *p, int *code, char **rest) ;
 
@@ -72,7 +72,8 @@ void logOrPrint (int level, FILE *fp, const char *fmt, ...)
     __attribute__((__format__(printf, 3, 4)));
 
 /* Error handling functions for use with warn and die. */
-void error_log_stderr_date(int len, const char *fmt, va_list args, int err);
+void error_log_stderr_date(int len, const char *fmt, va_list args, int err)
+    __attribute__((__format__(printf, 2, 0)));
 
 /* Do cleanup and then abort, for use with die. */
 void dump_core(void)
