@@ -1,5 +1,4 @@
-/*  $Id: ftello.c,v 1.1.2.2 2000/08/05 22:49:28 rra Exp $
-**
+/*
 **  Replacement for a missing ftello.
 **
 **  ftello is a version of ftell that returns an off_t instead of a long.
@@ -13,9 +12,6 @@
 #include "config.h"
 #include "clibrary.h"
 
-/* Make sure we get the system ftell (clibrary.h #defines it to ftello). */
-#undef ftell
-
 #if HAVE_LARGE_FPOS_T
 
 off_t
