@@ -1,14 +1,10 @@
-/*  $Id: getmodaddr.c 7585 2006-11-21 09:37:51Z eagle $
-**
-*/
-
 #include "config.h"
 #include "clibrary.h"
 #include <errno.h>
 
 #include "inn/innconf.h"
 #include "inn/libinn.h"
-#include "nntp.h"
+#include "inn/nntp.h"
 #include "inn/paths.h"
 
 
@@ -40,6 +36,7 @@ static FILE *
 GMA_listopen(int fd, FILE *FromServer, FILE *ToServer, const char *request)
 {
     char	buff[BUFSIZ];
+    char        expectedanswer[BUFSIZ];
     char	*p;
     int		oerrno;
     FILE	*F;
@@ -50,14 +47,16 @@ GMA_listopen(int fd, FILE *FromServer, FILE *ToServer, const char *request)
 
     /* Send a LIST command to and capture the output. */
     if (request == NULL)
-	fprintf(ToServer, "list moderators\r\n");
+	fprintf(ToServer, "LIST MODERATORS\r\n");
     else
-	fprintf(ToServer, "list %s\r\n", request);
+	fprintf(ToServer, "LIST %s\r\n", request);
     fflush(ToServer);
 
+    snprintf(expectedanswer, sizeof(expectedanswer), "%d", NNTP_OK_LIST);
+
     /* Get the server's reply to our command. */
     if (fgets(buff, sizeof buff, FromServer) == NULL
-     || strncmp(buff, NNTP_LIST_FOLLOWS, strlen(NNTP_LIST_FOLLOWS)) != 0) {
+     || strncmp(buff, expectedanswer, strlen(expectedanswer)) != 0) {
 	oerrno = errno;
         fclose(F);
 	GMAclose();
@@ -87,6 +86,48 @@ GMA_listopen(int fd, FILE *FromServer, FILE *ToServer, const char *request)
     return NULL;
 }
 
+/*
+**  Check if the argument is a valid submission template according to RFC 6048.
+**  At least, make sure it does not contain "%" except as part of "%s" or "%%",
+**  and that only one occurrence of "%s" exists, if any.
+*/
+static bool
+IsValidSubmissionTemplate(const char *string)
+{
+    bool found = false;
+    const char *p;
+
+    /* Not NULL. */
+    if (string == NULL)
+        return false;
+
+    p = string;
+
+    while ((p = strchr(p, '%')) != NULL) {
+        /* Look at the next character. */
+        p++;
+
+        /* Skip "%%". */
+        if (*p == '%') {
+           p++;
+           continue;
+        }
+
+        /* Invalid template if not "%s". */
+        if (*p != 's')
+            return false;
+
+        /* Invalid template if another "%s". */
+        if (found)
+            return false;
+
+        found = true;
+    }
+
+    return true;
+}
+
+
 /*
 **  Read the moderators file, looking for a moderator.
 */
@@ -121,7 +162,7 @@ GetModeratorAddress(FILE *FromServer, FILE *ToServer, char *group,
         GMApathname = concatpath(innconf->pathtmp, INN_PATH_TEMPMODERATORS);
         fd = mkstemp(GMApathname);
         if (fd >= 0)
-            GMAfp = GMA_listopen(fd, FromServer, ToServer, "moderators");
+            GMAfp = GMA_listopen(fd, FromServer, ToServer, "MODERATORS");
         else
             GMAfp = NULL;
 
@@ -155,12 +196,14 @@ GetModeratorAddress(FILE *FromServer, FILE *ToServer, char *group,
 		for (p = name; *p; p++)
 		    if (*p == '.')
 			*p = '-';
-		snprintf(address, sizeof(address), save, name);
-		break;
+                if (IsValidSubmissionTemplate(save)) {
+                    snprintf(address, sizeof(address), save, name);
+                    break;
+                }
 	    }
 	}
 
-	 GMAclose();
+	GMAclose();
 	if (address[0])
 	    return address;
     }
@@ -172,6 +215,12 @@ GetModeratorAddress(FILE *FromServer, FILE *ToServer, char *group,
     for (p = name; *p; p++)
 	if (*p == '.')
 	    *p = '-';
-    snprintf(address, sizeof(address), save, name);
+
+    if (IsValidSubmissionTemplate(save)) {
+        snprintf(address, sizeof(address), save, name);
+    } else {
+        return NULL;
+    }
+
     return address;
 }
