@@ -1,5 +1,4 @@
-/* $Id: inet_ntop.c 9768 2014-12-07 21:22:57Z iulius $
- *
+/*
  * Replacement for a missing inet_ntop.
  *
  * Provides an implementation of inet_ntop that only supports IPv4 addresses
@@ -9,33 +8,35 @@
  * host supports IPv6.
  *
  * The canonical version of this file is maintained in the rra-c-util package,
- * which can be found at <http://www.eyrie.org/~eagle/software/rra-c-util/>.
+ * which can be found at <https://www.eyrie.org/~eagle/software/rra-c-util/>.
  *
  * Written by Russ Allbery <eagle@eyrie.org>
+ * Copyright 2005, 2020 Russ Allbery <eagle@eyrie.org>
+ * Copyright 2008, 2011, 2013-2014
+ *     The Board of Trustees of the Leland Stanford Junior University
  *
- * The authors hereby relinquish any claim to any copyright that they may have
- * in this work, whether granted under contract or by operation of law or
- * international treaty, and hereby commit to the public, at large, that they
- * shall not, at any time in the future, seek to enforce any copyright in this
- * work against any person or entity, or prevent any person or entity from
- * copying, publishing, distributing or creating derivative works of this
- * work.
+ * Copying and distribution of this file, with or without modification, are
+ * permitted in any medium without royalty provided the copyright notice and
+ * this notice are preserved.  This file is offered as-is, without any
+ * warranty.
+ *
+ * SPDX-License-Identifier: FSFAP
  */
 
 #include "config.h"
-#include "clibrary.h"
 #include "portable/socket.h"
+#include "clibrary.h"
 
 #include <errno.h>
 
 /* This may already be defined by the system headers. */
 #ifndef INET_ADDRSTRLEN
-# define INET_ADDRSTRLEN 16
+#    define INET_ADDRSTRLEN 16
 #endif
 
 /* Systems old enough to not support inet_ntop may not have this either. */
 #ifndef EAFNOSUPPORT
-# define EAFNOSUPPORT EDOM
+#    define EAFNOSUPPORT EDOM
 #endif
 
 /*
@@ -43,7 +44,8 @@
  * the system version.
  */
 #if TESTING
-# define inet_ntop test_inet_ntop
+#    undef inet_ntop
+#    define inet_ntop test_inet_ntop
 const char *test_inet_ntop(int, const void *, char *, socklen_t);
 #endif
 
@@ -62,12 +64,14 @@ inet_ntop(int af, const void *src, char *dst, socklen_t size)
         return NULL;
     }
     p = src;
+    /* clang-format off */
     status = snprintf(dst, size, "%u.%u.%u.%u",
                       (unsigned int) (p[0] & 0xff),
                       (unsigned int) (p[1] & 0xff),
                       (unsigned int) (p[2] & 0xff),
                       (unsigned int) (p[3] & 0xff));
-    if (status < 0)
+    /* clang-format on */
+    if (status < 0 || (size_t) status >= (size_t) size)
         return NULL;
     return dst;
 }
