@@ -1,7 +1,3 @@
-/*  $Id: localopen.c 8469 2009-05-16 12:43:53Z iulius $
-**
-*/
-
 #include "config.h"
 #include "clibrary.h"
 #include <errno.h>
@@ -9,11 +5,11 @@
 
 #include "inn/innconf.h"
 #include "inn/libinn.h"
-#include "nntp.h"
+#include "inn/nntp.h"
 #include "inn/paths.h"
 
 #ifdef HAVE_UNIX_DOMAIN_SOCKETS
-# include <sys/un.h>
+# include "portable/socket-unix.h"
 #endif
 
 
