@@ -1,5 +1,4 @@
-/*  $Id: lockfile.c,v 1.11 2002/12/20 00:19:58 rra Exp $
-**
+/*
 **  Lock a file or a range in a file.
 **
 **  Provides inn_lock_file and inn_lock_range functions to lock or unlock a
@@ -12,7 +11,7 @@
 #include <errno.h>
 #include <fcntl.h>
 
-#include "libinn.h"
+#include "inn/libinn.h"
 
 bool
 inn_lock_file(int fd, enum inn_locktype type, bool block)
