@@ -1,5 +1,4 @@
-/* $Id: newsuser.c 9854 2015-05-14 13:12:36Z iulius $
- *
+/*
  * Ensure running as "news" user/group.
  *
  * By Ivan Shmakov, 2007.
@@ -69,14 +68,16 @@ ensure_news_user(bool may_setuid)
     if (geteuid() == 0) {
         if (! may_setuid) {
             /* NB:  mustn't be run as root, unless "may_setuid" is true. */
-            die("must be run as %s, not as root", innconf->runasuser);
+            die("must be run as %s, not as root",
+                innconf != NULL ? innconf->runasuser : RUNASUSER);
         }
         if (setuid(uid) < 0) {
             sysdie("failed to setuid");
         }
     }
     if (geteuid() != uid || getuid() != uid) {
-        die("must be run as %s", innconf->runasuser);
+        die("must be run as %s",
+            innconf != NULL ? innconf->runasuser : RUNASUSER);
     }
 }
 
@@ -95,7 +96,8 @@ ensure_news_grp(bool may_setgid)
         }
     }
     if (getegid() != gid || getgid() != gid) {
-        die ("must be run as %s group", innconf->runasgroup);
+        die ("must be run as %s group",
+             innconf != NULL ? innconf->runasgroup : RUNASGROUP);
     }
 }
 
