@@ -1,14 +1,14 @@
-/*  $Revision: 1.2 $
-**
+/*
 **  Radix-32 strings divide a number into five-bit nibbles and use the
 **  alphabet 0..9a..v to represent 0..32.
 */
-#include <stdio.h>
-#include <sys/types.h>
-#include "configdata.h"
+
+#include "config.h"
 #include "clibrary.h"
 #include <time.h>
 
+#include "inn/libinn.h"
+
 
 static char	ALPHABET[] =
     "0123456789abcdefghijklmnopqrstuv";
@@ -48,10 +48,10 @@ void Radix32(unsigned long l, char *buff)
 */
 unsigned long
 Decode32(p)
-    register char		*p;
+    char		*p;
 {
-    register unsigned long	l;
-    register char		*cp;
+    unsigned long	l;
+    char		*cp;
 
     for (l = 0; *p; p++) {
 	if ((cp = strchr(ALPHABET, *p)) == NULL)
