@@ -1,15 +1,13 @@
-/*  $Revision: 1.4 $
-**
-*/
-#include <stdio.h>
-#include <sys/types.h>
-#include "configdata.h"
+#include "config.h"
 #include "clibrary.h"
-#include "macros.h"
+#include "inn/libinn.h"
 
 #ifdef HAVE_GETRUSAGE
 
-#include <sys/time.h>
+#ifdef HAVE_SYS_TIME_H
+# include <sys/time.h>
+#endif
+#include <time.h>
 #include <sys/resource.h>
 
 #define TIMEVALasDOUBLE(t)	\
