@@ -1,5 +1,4 @@
-/*  $Id: timer.c 9790 2015-01-21 18:49:11Z iulius $
-**
+/*
 **  Timer functions, to gather profiling data.
 **
 **  These functions log profiling information about where the server spends
@@ -39,9 +38,13 @@
 
 #include "config.h"
 #include "clibrary.h"
-#include "portable/time.h"
 #include <syslog.h>
 
+#ifdef HAVE_SYS_TIME_H
+# include <sys/time.h>
+#endif
+#include <time.h>
+
 #include "inn/messages.h"
 #include "inn/timer.h"
 #include "inn/libinn.h"
@@ -110,6 +113,7 @@ static unsigned long
 TMRgettime(bool reset)
 {
     unsigned long now;
+    long usec, msec;
     struct timeval tv;
 
     /* The time of the last summary, used as a base for times returned by
@@ -121,8 +125,10 @@ TMRgettime(bool reset)
     static struct timeval base;
 
     gettimeofday(&tv, NULL);
-    now = (tv.tv_sec - base.tv_sec) * 1000;
-    now += (tv.tv_usec - base.tv_usec) / 1000;
+    now = (unsigned long)(tv.tv_sec - base.tv_sec) * 1000u;
+    usec = tv.tv_usec - base.tv_usec; /* maybe negative */
+    msec = usec / 1000;               /* still maybe negative */
+    now += (unsigned long)msec;
     if (reset)
         base = tv;
     return now;
@@ -315,21 +321,39 @@ TMRsumone(const char *const *labels, struct timer *timer, char *buf,
 {
     struct timer *node;
     size_t off = 0;
+    int rc;
 
     /* This results in "child/parent nn(nn)" instead of the arguably more
        intuitive "parent/child" but it's easy.  Since we ensure sane snprintf 
        semantics, it's safe to defer checking for overflow until after
        formatting all of the timer data. */
-    for (node = timer; node != NULL; node = node->parent)
-        off += snprintf(buf + off, len - off, "%s/",
+    for (node = timer; node != NULL; node = node->parent) {
+        rc = snprintf(buf + off, len - off, "%s/",
                         TMRlabel(labels, node->id));
-    off--;
-    off += snprintf(buf + off, len - off, " %lu(%lu) ", timer->total,
+        if (rc < 0) {
+            /* Do nothing. */
+        } else if ((size_t)rc >= len - off) {
+            off = len;
+        } else {
+            off += rc;
+        }
+    }
+    if (off > 0)
+        off--;
+
+    rc = snprintf(buf + off, len - off, " %lu(%lu) ", timer->total,
                     timer->count);
+    if (rc < 0) {
+        /* Do nothing. */
+    } else if ((size_t)rc >= len - off) {
+        off = len;
+    } else {
+        off += rc;
+    }
     if (off == len) {
         warn("timer log too long while processing %s",
              TMRlabel(labels, timer->id));
-        return 0;
+        return off;
     }
 
     timer->total = 0;
@@ -352,6 +376,7 @@ TMRsummary(const char *prefix, const char *const *labels)
     char *buf;
     unsigned int i;
     size_t len, off;
+    int rc;
 
     /* To find the needed buffer size, note that a 64-bit unsigned number can 
        be up to 20 digits long, so each timer can be 52 characters.  We also
@@ -362,14 +387,33 @@ TMRsummary(const char *prefix, const char *const *labels)
        buffer isn't large enough it will just result in logged errors. */
     len = 52 * timer_count + 27 + (prefix == NULL ? 0 : strlen(prefix)) + 1;
     buf = xmalloc(len);
+    off = 0;
     if (prefix == NULL)
-        off = 0;
+        rc = 0;
     else
-        off = snprintf(buf, len, "%s ", prefix);
-    off += snprintf(buf + off, len - off, "time %lu ", TMRgettime(true));
-    for (i = 0; i < timer_count; i++)
-        if (timers[i] != NULL)
+        rc = snprintf(buf, len, "%s ", prefix);
+    if (rc < 0) {
+        /* Do nothing. */
+    } else if ((size_t)rc >= len) {
+        off = len;
+    } else {
+        off += rc;
+    }
+
+    rc = snprintf(buf + off, len - off, "time %lu ", TMRgettime(true));
+    if (rc < 0) {
+        /* Do nothing. */
+    } else if ((size_t)rc >= len - off) {
+        off = len;
+    } else {
+        off += rc;
+    }
+
+    for (i = 0; i < timer_count; i++) {
+        if (timers[i] != NULL) {
             off += TMRsumone(labels, timers[i], buf + off, len - off);
-    syslog(LOG_NOTICE, "%s", buf);
+        }
+    }
+    notice("%s", buf);
     free(buf);
 }
