@@ -1,5 +1,4 @@
-/*  $Id: article.c 9860 2015-05-14 13:32:30Z iulius $
-**
+/*
 **  Article-related routines.
 */
 
@@ -20,7 +19,8 @@
 #include "tls.h"
 #include "cache.h"
 
-#ifdef HAVE_SSL
+extern bool laxmid;
+#ifdef HAVE_OPENSSL
 extern SSL *tls_conn;
 #endif 
 
@@ -64,6 +64,25 @@ PushIOvHelper(struct iovec* vec, int* countp)
 
     TMRstart(TMR_NNTPWRITE);
 
+#if defined(HAVE_ZLIB)
+    if (compression_layer_on) {
+        int i;
+
+        for (i = 0; i < *countp; i++) {
+            if (i+1 == *countp) {
+                /* Time to flush the compressed output stream. */
+                zstream_flush_needed = true;
+            }
+            write_buffer(vec[i].iov_base, vec[i].iov_len);
+        }
+
+        *countp = 0;
+
+        TMRstop(TMR_NNTPWRITE);
+        return;
+    }
+#endif /* HAVE_ZLIB */
+
 #ifdef HAVE_SASL
     if (sasl_conn && sasl_ssf) {
 	int i;
@@ -74,27 +93,28 @@ PushIOvHelper(struct iovec* vec, int* countp)
     } else {
 #endif /* HAVE_SASL */
 
-#ifdef HAVE_SSL
+#ifdef HAVE_OPENSSL
 	if (tls_conn) {
 Again:
 	    result = SSL_writev(tls_conn, vec, *countp);
 	    switch (SSL_get_error(tls_conn, result)) {
 	    case SSL_ERROR_NONE:
-	    case SSL_ERROR_SYSCALL:
 		break;
 	    case SSL_ERROR_WANT_WRITE:
 		goto Again;
 		break;
-	    case SSL_ERROR_SSL:
-		SSL_shutdown(tls_conn);
+            case SSL_ERROR_ZERO_RETURN:
+                SSL_shutdown(tls_conn);
+                /* fallthrough */
+            case SSL_ERROR_SSL:
+            case SSL_ERROR_SYSCALL:
+                /* SSL_shutdown() must not be called. */
 		tls_conn = NULL;
 		errno = ECONNRESET;
 		break;
-	    case SSL_ERROR_ZERO_RETURN:
-		break;
 	    }
 	} else
-#endif /* HAVE_SSL */
+#endif /* HAVE_OPENSSL */
 	    result = xwritev(STDOUT_FILENO, vec, *countp);
 
 #ifdef HAVE_SASL
@@ -196,6 +216,12 @@ static int		highwater = 0;
 static void
 PushIOb(void)
 {
+#if defined(HAVE_ZLIB)
+    /* Last line of a multi-line data block response.
+     * Time to flush the compressed output stream. */
+    zstream_flush_needed = true;
+#endif /* HAVE_ZLIB */
+
     write_buffer(_IO_buffer_, highwater);
     highwater = 0;
 }
@@ -613,7 +639,7 @@ CMDfetch(int ac, char *av[])
     ARTNUM		tart;
     bool                final = false;
 
-    mid = (ac > 1 && IsValidMessageID(av[1], true));
+    mid = (ac > 1 && IsValidMessageID(av[1], true, laxmid));
 
     /* Check the syntax of the arguments first. */
     if (ac > 1 && !IsValidArticleNumber(av[1])) {
@@ -676,7 +702,6 @@ CMDfetch(int ac, char *av[])
                   PERMcanauthenticate ? NNTP_FAIL_AUTH_NEEDED : NNTP_ERR_ACCESS);
 	    return;
 	}
-	tart=art;
 	Reply("%d %lu %s %s\r\n", what->ReplyCode, art, av[1], what->Item);
 	if (what->Type != STstat) {
 	    ARTsendmmap(what->Type);
@@ -909,7 +934,7 @@ CMDover(int ac, char *av[])
     bool                xover, mid;
 
     xover = (strcasecmp(av[0], "XOVER") == 0);
-    mid = (ac > 1 && IsValidMessageID(av[1], true));
+    mid = (ac > 1 && IsValidMessageID(av[1], true, laxmid));
 
     if (mid && !xover) {
         /* FIXME:  We still do not support OVER MSGID, sorry! */
@@ -1129,7 +1154,7 @@ CMDpat(int ac, char *av[])
     bool                hdr, mid;
 
     hdr = (strcasecmp(av[0], "HDR") == 0);
-    mid = (ac > 2 && IsValidMessageID(av[2], true));
+    mid = (ac > 2 && IsValidMessageID(av[2], true, laxmid));
 
     /* Check the syntax of the arguments first. */
     if (ac > 2 && !IsValidRange(av[2])) {
