@@ -1,8 +1,12 @@
+/*
+**  Message-ID to storage token cache.
+*/
+
 #ifndef CACHE_H
 #define CACHE_H
 
-#include "libinn.h"
-#include "storage.h"
+#include "inn/libinn.h"
+#include "inn/storage.h"
 
 BEGIN_DECLS
 
