@@ -1,5 +1,4 @@
-/*  $Id: list.c 8409 2009-04-11 22:13:41Z iulius $
-**
+/*
 **  LIST commands.
 */
 
@@ -24,49 +23,55 @@ static void cmd_list_schema(LISTINFO *lp, int ac, char *av[]);
 static void cmd_list_headers(LISTINFO *lp, int ac, char *av[]);
 
 static LISTINFO		INFOactive = {
-    "ACTIVE", INN_PATH_ACTIVE, NULL, true, "active newsgroups",
-    "Newsgroups in form \"group high low flags\""
+    "ACTIVE", INN_PATH_ACTIVE, NULL, true, "list of active newsgroups",
+    "Newsgroups in form \"group high low status\""
 };
 static LISTINFO		INFOactivetimes = {
-    "ACTIVE.TIMES", INN_PATH_ACTIVETIMES, NULL, false, "creation times",
-    "Group creations in form \"name time who\""
+    "ACTIVE.TIMES", INN_PATH_ACTIVETIMES, NULL, false, "list of newsgroup creation times",
+    "Newsgroup creation times in form \"group time who\""
+};
+static LISTINFO         INFOcounts = {
+    "COUNTS", INN_PATH_ACTIVE, NULL, true, "list of active newsgroups with estimated counts",
+    "Newsgroups in form \"group high low count status\""
 };
 static LISTINFO		INFOdistribs = {
-    "DISTRIBUTIONS", INN_PATH_NNRPDIST, NULL, false, "newsgroup distributions",
-    "Distributions in form \"area description\""
+    "DISTRIBUTIONS", INN_PATH_NNRPDIST, NULL, false, "list of newsgroup distributions",
+    "Distributions in form \"distribution description\""
 };
 static LISTINFO         INFOheaders = {
-    "HEADERS", NULL, cmd_list_headers, true, "supported headers and metadata",
+    "HEADERS", NULL, cmd_list_headers, true, "list of supported headers and metadata items",
     "Headers and metadata items supported"
 };
-static LISTINFO               INFOsubs = {
+static LISTINFO         INFOsubs = {
     "SUBSCRIPTIONS", INN_PATH_NNRPSUBS, NULL, false,
-    "automatic group subscriptions", "Subscriptions in form \"group\""
+    "list of recommended newsgroup subscriptions",
+    "Recommended subscriptions in form \"group\""
 };
 static LISTINFO		INFOdistribpats = {
-    "DISTRIB.PATS", INN_PATH_DISTPATS, NULL, false, "distribution patterns",
-    "Default distributions in form \"weight:pattern:value\""
+    "DISTRIB.PATS", INN_PATH_DISTPATS, NULL, false, "list of distribution patterns",
+    "Default distributions in form \"weight:group-pattern:distribution\""
 };
 static LISTINFO		INFOgroups = {
-    "NEWSGROUPS", INN_PATH_NEWSGROUPS, NULL, true, "newsgroup descriptions",
-    "Descriptions in form \"group description\""
+    "NEWSGROUPS", INN_PATH_NEWSGROUPS, NULL, true, "list of newsgroup descriptions",
+    "Newsgroup descriptions in form \"group description\""
 };
 static LISTINFO		INFOmoderators = {
-    "MODERATORS", INN_PATH_MODERATORS, NULL, false, "moderator patterns",
-    "Newsgroup moderators in form \"group-pattern:mail-address-pattern\""
+    "MODERATORS", INN_PATH_MODERATORS, NULL, false, "list of submission templates",
+    "Newsgroup moderators in form \"group-pattern:submission-template\""
 };
 static LISTINFO		INFOschema = {
     "OVERVIEW.FMT", NULL, cmd_list_schema, true, "overview format",
     "Order of fields in overview database"
 };
 static LISTINFO		INFOmotd = {
-    "MOTD", INN_PATH_MOTD, NULL, false, "motd",
-    "Message of the day text"
+    "MOTD", INN_PATH_MOTD_NNRPD, NULL, false, "message of the day",
+    "Message of the day text in UTF-8"
 };
 
 static LISTINFO *info[] = {
     &INFOactive,
     &INFOactivetimes,
+    &INFOcounts,
     &INFOdistribs,
     &INFOheaders,
     &INFOsubs,
@@ -108,7 +113,7 @@ cmd_list_headers(LISTINFO *lp, int ac, char *av[])
     bool range;
 
     range = (ac > 2 && strcasecmp(av[2], "RANGE") == 0);
-    
+
     if (ac > 2 && (strcasecmp(av[2], "MSGID") != 0)
         && !range) {
         Reply("%d Syntax error in arguments\r\n", NNTP_ERR_SYNTAX);
@@ -144,16 +149,19 @@ CMD_list_single(char *group)
         if (!PERMmatch(PERMreadlist, grplist))
             return false;
     }
-    if (OVgroupstats(group, &lo, &hi, &count, &flag) && flag != '=') {
-        /* Convert flags to standardized ones, if possible. */
-        if (flag == 'j' || flag == 'x')
-            flag = 'n';
+    if (OVgroupstats(group, &lo, &hi, &count, &flag) && flag != NF_FLAG_ALIAS) {
+        /* When the connected user has the right to locally post, mention it. */
+        if (PERMaccessconf->locpost && (flag == NF_FLAG_IGNORE
+                                        || flag == NF_FLAG_JUNK
+                                        || flag == NF_FLAG_NOLOCAL))
+            flag = NF_FLAG_OK;
         /* When a newsgroup is empty, the high water mark should be one less
          * than the low water mark according to RFC 3977. */
         if (count == 0)
             lo = hi + 1;
         Reply("%d %s\r\n", NNTP_OK_LIST, INFOactive.Format);
-        Printf("%s %010u %010u %c\r\n", group, hi, lo, flag);
+        Printf("%s %0*u %0*u %c\r\n", group, ARTNUMPRINTSIZE, hi,
+               ARTNUMPRINTSIZE, lo, flag);
         Printf(".\r\n");
         return true;
     }
@@ -177,8 +185,10 @@ CMDlist(int ac, char *av[])
     char		*wildarg = NULL;
     char		savec;
     unsigned int i;
+    int         lo, hi, count, flag;
 
     p = av[1];
+
     /* LIST ACTIVE is the default LIST command.  If a keyword is provided,
      * we check whether it is defined. */
     if (p == NULL) {
@@ -192,6 +202,7 @@ CMDlist(int ac, char *av[])
 	    }
 	}
     }
+
     /* If no defined LIST keyword is found, we return. */
     if (lp == NULL) {
         Reply("%d Unknown LIST keyword\r\n", NNTP_ERR_SYNTAX);
@@ -205,15 +216,16 @@ CMDlist(int ac, char *av[])
             if (CMD_list_single(wildarg))
 		return;
 	}
-    } else if (lp == &INFOgroups || lp == &INFOactivetimes
-               || lp == &INFOheaders) {
+    } else if (lp == &INFOgroups || lp == &INFOactivetimes || lp == &INFOcounts
+               || lp == &INFOheaders || lp == &INFOsubs) {
 	if (ac == 3)
 	    wildarg = av[2];
     }
-    /* Three arguments can be passed only when ACTIVE, ACTIVE.TIMES,
-     * HEADERS or NEWSGROUPS keywords are used. */
+
+    /* Three arguments can be passed only when ACTIVE, ACTIVE.TIMES, COUNTS
+     * HEADERS, NEWSGROUPS or SUBSCRIPTIONS keywords are used. */
     if (ac > 2 && !wildarg) {
-        Reply("%d Unexpected wildmat\r\n", NNTP_ERR_SYNTAX);
+        Reply("%d Unexpected wildmat or argument\r\n", NNTP_ERR_SYNTAX);
         return;
     }
 
@@ -224,6 +236,7 @@ CMDlist(int ac, char *av[])
     }
 
     path = innconf->pathetc;
+
     /* The active, active.times and newsgroups files are in pathdb. */
     if ((strstr(lp->File, "active") != NULL) ||
 	(strstr(lp->File, "newsgroups") != NULL))
@@ -232,7 +245,7 @@ CMDlist(int ac, char *av[])
     qp = QIOopen(path);
     free(path);
     if (qp == NULL) {
-        Reply("%d No list of %s available\r\n",
+        Reply("%d No %s available\r\n",
               NNTP_ERR_UNAVAILABLE, lp->Items);
         /* Only the active and newsgroups files are required. */
         if (lp->Required || errno != ENOENT) {
@@ -243,7 +256,7 @@ CMDlist(int ac, char *av[])
     }
 
     Reply("%d %s\r\n", NNTP_OK_LIST, lp->Format);
-    if (!PERMspecified) {
+    if (!PERMspecified && lp != &INFOmotd) {
 	/* Optimize for unlikely case of no permissions and false default. */
 	QIOclose(qp);
 	Printf(".\r\n");
@@ -255,14 +268,20 @@ CMDlist(int ac, char *av[])
 
     /* Read lines, ignore long ones. */
     while ((p = QIOread(qp)) != NULL) {
-	if (lp == &INFOmotd) {
-	    Printf("%s\r\n", p);
-	    continue;
-	}
-	if (p[0] == '.' && p[1] == '\0') {
-	    syslog(L_ERROR, "%s single dot in %s", Client.host, lp->File);
-	    continue;
-	}
+        /* Check that the output does not break the NNTP protocol. */
+        if (p[0] == '.' && p[1] != '.') {
+            syslog(L_ERROR, "%s bad dot-stuffing in %s", Client.host, lp->File);
+            continue;
+        }
+        if (lp == &INFOmotd) {
+            if (is_valid_utf8(p)) {
+                Printf("%s\r\n", p);
+            } else {
+                syslog(L_ERROR, "%s bad encoding in %s (UTF-8 expected)",
+                       Client.host, lp->File);
+            }
+            continue;
+        }
 	/* Matching patterns against patterns is not that
 	 * good but it is better than nothing... */
 	if (lp == &INFOdistribpats) {
@@ -284,8 +303,14 @@ CMDlist(int ac, char *av[])
 	    continue;
 	}
 	if (lp == &INFOdistribs || lp == &INFOmoderators) {
-	    if (*p != '\0' && *p != '#' && *p != ';' && *p != ' ')
-		Printf("%s\r\n", p);
+            if (*p != '\0' && *p != '#' && *p != ';' && *p != ' ') {
+                if (is_valid_utf8(p)) {
+                    Printf("%s\r\n", p);
+                } else if (lp == &INFOdistribs) {
+                    syslog(L_ERROR, "%s bad encoding in %s (UTF-8 expected)",
+                           Client.host, lp->File);
+                }
+            }
 	    continue;
 	}
 	savec = '\0';
@@ -303,22 +328,53 @@ CMDlist(int ac, char *av[])
 	    if (!PERMmatch(PERMreadlist, grplist))
 		continue;
 	}
+
         /* Check whether the newsgroup matches the wildmat pattern,
          * if given. */
 	if (wildarg && !uwildmat(p, wildarg))
 	    continue;
 
+        if (lp == &INFOcounts) {
+            if (OVgroupstats(p, &lo, &hi, &count, &flag)) {
+                /* When a newsgroup is empty, the high water mark should be
+                 * one less than the low water mark according to RFC 3977. */
+                if (count == 0)
+                    lo = hi + 1;
+
+                if (flag != NF_FLAG_ALIAS) {
+                    Printf("%s %u %u %u %c\r\n", p, hi, lo, count,
+                           PERMaccessconf->locpost
+                           && (flag == NF_FLAG_IGNORE
+                               || flag == NF_FLAG_JUNK
+                               || flag == NF_FLAG_NOLOCAL)
+                           ? NF_FLAG_OK : flag);
+                } else if (savec != '\0') {
+                    *save = savec;
+
+                    if ((q = strrchr(p, NF_FLAG_ALIAS)) != NULL) {
+                        *save = '\0';
+                        Printf("%s %u %u %u %s\r\n", p, hi, lo, count, q);
+                    }
+                }
+            }
+
+            continue;
+        }
+
 	if (savec != '\0')
 	    *save = savec;
 
         if (lp == &INFOactive) {
-            /* Convert flags to standardized ones, if possible. */
-            if ((q = strrchr(p, ' ')) != NULL) {
+            /* When the connected user has the right to locally post, mention it. */
+            if (PERMaccessconf->locpost && (q = strrchr(p, ' ')) != NULL) {
                 q++;
-                if (*q == 'j' || *q == 'x')
-                    *q = 'n';
+                if (*q == NF_FLAG_IGNORE
+                    || *q == NF_FLAG_JUNK
+                    || *q == NF_FLAG_NOLOCAL)
+                    *q = NF_FLAG_OK;
             }
         }
+
 	Printf("%s\r\n", p);
     }
     QIOclose(qp);
