@@ -1,5 +1,4 @@
-/*  $Id: newnews.c 8519 2009-06-20 12:20:03Z iulius $
-**
+/*
 **  The NEWNEWS command.
 */
 #include "config.h"
@@ -21,7 +20,7 @@ FindHeader(ARTHANDLE *art, const char **pp, const char **qp,
   const char *p, *p1, *q;
   bool Nocr = true;
 
-  p = wire_findheader(art->data, art->len, hdr);
+  p = wire_findheader(art->data, art->len, hdr, true);
   if (p == NULL)
     return false;
   q = p;
@@ -117,7 +116,7 @@ HaveSeen(bool AllGroups, char *group, char **groups, char **xrefs)
   list[1] = NULL;
   for ( ; *xrefs; xrefs++) {
     list[0] = *xrefs;
-    if ((!AllGroups && PERMmatch(groups, list)) && (!PERMspecified || (PERMspecified && PERMmatch(PERMreadlist, list)))) {
+    if ((!AllGroups && PERMmatch(groups, list)) && (!PERMspecified || PERMmatch(PERMreadlist, list))) {
       if (!strcmp(*xrefs, group))
 	return false;
       else
@@ -165,11 +164,11 @@ process_newnews(char *group, bool AllGroups, time_t date)
 	    time(&now);
 	    /* Move the start time back nfsreaderdelay seconds
              * as we are an NFS reader. */
-	    if (date >= innconf->nfsreaderdelay)
+	    if (date >= (time_t) innconf->nfsreaderdelay)
 		date -= innconf->nfsreaderdelay;
 	}
 	while (OVsearch(handle, &artnum, &data, &len, &token, &arrived)) {
-	    if (innconf->nfsreader && arrived + innconf->nfsreaderdelay > now)
+	    if (innconf->nfsreader != 0 && (time_t) (arrived + innconf->nfsreaderdelay) > now)
 		continue;
 	    if (len == 0 || date > arrived)
 		continue;
@@ -285,7 +284,7 @@ CMDnewnews(int ac, char *av[])
   }
 
   /* Make other processes happier if someone uses NEWNEWS. */
-  if (innconf->nicenewnews > 0) {
+  if (innconf->nicenewnews != 0) {
       nice(innconf->nicenewnews);
       innconf->nicenewnews = 0;
   }
@@ -293,7 +292,7 @@ CMDnewnews(int ac, char *av[])
   if (strcspn(av[1], "\\!*[?]") == strlen(av[1])) {
       /* Optimise case -- don't need to scan the active file pattern
        * matching. */
-      Reply("%s\r\n", NNTP_NEWNEWSOK);
+      Reply("%d New news follows\r\n", NNTP_OK_NEWNEWS);
       for (i = 0; groups[i]; ++i) {
 	  process_newnews(groups[i], AllGroups, date);
       }
@@ -313,7 +312,7 @@ CMDnewnews(int ac, char *av[])
       }
       free(path);
 
-      Reply("%s\r\n", NNTP_NEWNEWSOK);
+      Reply("%d New news follows\r\n", NNTP_OK_NEWNEWS);
 
       while ((p = QIOread(qp)) != NULL) {
 	  for (q = p; *q != '\0'; q++) {
