@@ -1,5 +1,4 @@
-/*  $Id: nnrpd.c 10229 2018-01-28 21:21:21Z iulius $
-**
+/*
 **  NNTP server for readers (NNRP) for InterNetNews.
 **
 **  This server doesn't do any real load-limiting, except for what has
@@ -7,7 +6,6 @@
 */
 
 #include "config.h"
-#include "portable/alloca.h"
 #include "clibrary.h"
 #include "portable/setproctitle.h"
 #include "portable/socket.h"
@@ -578,10 +576,10 @@ Address2Name(struct sockaddr *sa, socklen_t len, char *hostname, size_t size)
 
 
 /*
-**  Determine access rights of the client.
+**  Determine hostname and IP of the client, amongst other information.
 */
 static void
-StartConnection(unsigned short port)
+GetClientInfo(unsigned short port)
 {
     static const char *default_host_error = "unknown error";
     struct sockaddr_storage ssc, sss;
@@ -666,10 +664,6 @@ StartConnection(unsigned short port)
 #endif
 
     notice("%s (%s) connect - port %u", Client.host, Client.ip, port);
-
-    PERMgetinitialaccess(NNRPACCESS);
-    PERMgetaccess(true);
-    PERMgetpermissions();
 }
 
 
@@ -712,6 +706,7 @@ write_buffer(const char *buff, ssize_t len)
                 sysnotice("deflate() failed: %d; %s", r,
                           zstream_out->msg != NULL ? zstream_out->msg :
                           "no detail");
+                TMRstop(TMR_NNTPWRITE);
                 return;
             }
         } while (r == Z_OK && zstream_out->avail_out == 0);
@@ -738,6 +733,7 @@ write_buffer(const char *buff, ssize_t len)
                 sysnotice("sasl_encode() failed: %s; %s",
                           sasl_errstring(r, NULL, NULL),
                           ed != NULL ? ed : "no detail");
+                TMRstop(TMR_NNTPWRITE);
                 return;
             }
         } else
@@ -760,18 +756,19 @@ Again:
 	    r = SSL_write(tls_conn, out, outlen);
 	    switch (SSL_get_error(tls_conn, r)) {
 	    case SSL_ERROR_NONE:
-	    case SSL_ERROR_SYSCALL:
 		break;
 	    case SSL_ERROR_WANT_WRITE:
 		goto Again;
 		break;
+            case SSL_ERROR_ZERO_RETURN:
+                SSL_shutdown(tls_conn);
+                /* fallthrough */
 	    case SSL_ERROR_SSL:
-		SSL_shutdown(tls_conn);
+            case SSL_ERROR_SYSCALL:
+                /* SSL_shutdown() must not be called. */
 		tls_conn = NULL;
 		errno = ECONNRESET;
 		break;
-	    case SSL_ERROR_ZERO_RETURN:
-		break;
 	    }
 	} else
 #endif /* HAVE_OPENSSL */
@@ -1194,7 +1191,7 @@ main(int argc, char *argv[])
         FD_ZERO(&lfdset);
 
         for (j = 0; j < lfdcount; j++) {
-            if (listen(lfds[j], 128) < 0) {
+            if (listen(lfds[j], innconf->maxlisten) < 0) {
                 if (j != 0 && errno == EADDRINUSE)
                     continue;
                 syslog(L_ERROR, "can't listen to socket");
@@ -1326,7 +1323,10 @@ main(int argc, char *argv[])
     if (initialSSL) {
         tls_init();
         if (tls_start_servertls(0, 1) == -1) {
-            Reply("%d Encrypted TLS connection failed\r\n", NNTP_FAIL_TERMINATING);
+            GetClientInfo(ListenPort);
+            notice("%s failure to negotiate TLS session", Client.host);
+            Reply("%d Encrypted TLS connection failed\r\n",
+                  NNTP_FAIL_TERMINATING);
             ExitWithStats(1, false);
         }
         encryption_layer_on = true;
@@ -1349,7 +1349,9 @@ main(int argc, char *argv[])
             warn("cannot obtain system load");
         else {
             if ((unsigned long)(load[0] + 0.5) > innconf->nnrpdloadlimit) {
-                syslog(L_NOTICE, "load %.2f > %lu", load[0], innconf->nnrpdloadlimit);
+                GetClientInfo(ListenPort);
+                notice("%s load %.2f > %lu", Client.host,
+                       load[0], innconf->nnrpdloadlimit);
                 Reply("%d load at %.2f, try later\r\n", NNTP_FAIL_TERMINATING,
                       load[0]);
                 ExitWithStats(1, true);
@@ -1361,7 +1363,11 @@ main(int argc, char *argv[])
     xsignal(SIGPIPE, CatchPipe);
 
     /* Get permissions and see if we can talk to this client. */
-    StartConnection(ListenPort);
+    GetClientInfo(ListenPort);
+    PERMgetinitialaccess(NNRPACCESS);
+    PERMgetaccess(true);
+    PERMgetpermissions();
+
     if (!PERMcanread && !PERMcanpost && !PERMneedauth) {
 	syslog(L_NOTICE, "%s no_permission", Client.host);
 	Reply("%d You have no permission to talk.  Goodbye!\r\n",
@@ -1441,7 +1447,6 @@ main(int argc, char *argv[])
     /* Main dispatch loop. */
     for (timeout = innconf->initialtimeout, av = NULL, ac = 0; ;
 			timeout = clienttimeout) {
-        alloca_free();
 	TMRstart(TMR_NNTPWRITE);
 	fflush(stdout);
 	TMRstop(TMR_NNTPWRITE);
