@@ -1,5 +1,4 @@
-/*  $Id: post.h 8049 2008-09-26 19:45:03Z iulius $
-**
+/*
 **  NetNews Reading Protocol server.
 */
 
@@ -48,13 +47,11 @@ typedef struct _HEADER {
 #define HDR__LINES	      9
 #define HDR__SENDER	     10
 #define HDR__APPROVED	     11
-#define HDR__DISTRIBUTION    12
-#define HDR__EXPIRES	     13
-#define HDR__MESSAGEID	     14
-#define HDR__NNTPPOSTINGHOST 17
-#define HDR__XTRACE          21
-#define HDR__XCOMPLAINTSTO   22
-#define HDR__NNTPPOSTINGDATE 23
-#define HDR__CC		     33
-#define HDR__BCC	     34
-#define HDR__TO		     35
+#define HDR__DISTRIBUTION    13
+#define HDR__EXPIRES	     14
+#define HDR__MESSAGEID	     15
+#define HDR__INJECTION_DATE  26
+#define HDR__INJECTION_INFO  27
+#define HDR__CC		     34
+#define HDR__BCC	     35
+#define HDR__TO		     36
