@@ -1,17 +1,9 @@
-/*  $Id: tls.c 10198 2017-12-10 15:22:43Z iulius $
-**
-**  tls.c -- TLSv1 functions.
+/*
+**  tls.c -- TLS functions.
 **  Copyright (C) 2000 Kenichi Okada <okada@opaopa.org>.
 **
 **  Author:  Kenichi Okada <okada@opaopa.org>
 **  Created:  2000-02-22
-**
-**  [RFC 2246] "The TLS Protocol Version 1.0"
-**      by Christopher Allen <callen@certicom.com> and
-**      Tim Dierks <tdierks@certicom.com> (1999/01)
-**
-**  [RFC 2595] "Using TLS with IMAP, POP3 and ACAP"
-**      by Chris Newman <chris.newman@innosoft.com> (1999/06)
 */
 
 #include "config.h"
@@ -54,7 +46,7 @@ const char   *tls_cipher_name = NULL;
 int	tls_cipher_usebits = 0;
 int	tls_cipher_algbits = 0;
 
-
+/* Set this value higher (from 1 to 4) to obtain more logs. */
 int tls_loglevel = 0;
 
 
@@ -102,47 +94,72 @@ apps_ssl_info_callback(const SSL *s, int where, int ret)
 
 
 /*
-**  Hardcoded DH parameter files, from OpenSSL.
-**  For information on how these files were generated, see
-**  "Assigned Number for SKIP Protocols" 
-**  <http://www.skip-vpn.org/spec/numbers.html>.
+**  Now that safe prime groups have been supported since OpenSSL 1.1.0,
+**  these groups and the following functions to deal with them are only kept
+**  for older OpenSSL versions.
 */
-static const char file_dh512[] =
+#if OPENSSL_VERSION_NUMBER < 0x010100000L
+/*
+**  Hardcoded DH parameter files.
+**  These are pre-defined DH groups recommended by RFC 7919 (Appendix A),
+**  that have been audited and therefore supposed to be more
+**  resistant to attacks than ones randomly generated.
+**  They are FIPS-140 compliant and impractical to attack by construction.
+**
+**  There isn't any need to support user-specific files because of the
+**  safe choice of these domain parameters from RFC 7919.
+**  In case they appear to no longer be secure in the future, they'll
+**  be changed in a future INN release.
+*/
+static const char file_ffdhe2048[] = \
 "-----BEGIN DH PARAMETERS-----\n\
-MEYCQQD1Kv884bEpQBgRjXyEpwpy1obEAxnIByl6ypUM2Zafq9AKUJsCRtMIPWak\n\
-XUGfnHy9iUsiGSa6q6Jew1XpKgVfAgEC\n\
+MIIBCAKCAQEA//////////+t+FRYortKmq/cViAnPTzx2LnFg84tNpWp4TZBFGQz\n\
++8yTnc4kmz75fS/jY2MMddj2gbICrsRhetPfHtXV/WVhJDP1H18GbtCFY2VVPe0a\n\
+87VXE15/V8k1mE8McODmi3fipona8+/och3xWKE2rec1MKzKT0g6eXq8CrGCsyT7\n\
+YdEIqUuyyOP7uWrat2DX9GgdT0Kj3jlN9K5W7edjcrsZCwenyO4KbXCeAvzhzffi\n\
+7MA0BM0oNC9hkXL+nOmFg/+OTxIy7vKBg8P+OxtMb61zO7X8vC7CIAXFjvGDfRaD\n\
+ssbzSibBsu/6iGtCOGEoXJf//////////wIBAg==\n\
 -----END DH PARAMETERS-----\n";
 
-static const char file_dh1024[] =
+static const char file_ffdhe4096[] = \
 "-----BEGIN DH PARAMETERS-----\n\
-MIGHAoGBAPSI/VhOSdvNILSd5JEHNmszbDgNRR0PfIizHHxbLY7288kjwEPwpVsY\n\
-jY67VYy4XTjTNP18F1dDox0YbN4zISy1Kv884bEpQBgRjXyEpwpy1obEAxnIByl6\n\
-ypUM2Zafq9AKUJsCRtMIPWakXUGfnHy9iUsiGSa6q6Jew1XpL3jHAgEC\n\
+MIICCAKCAgEA//////////+t+FRYortKmq/cViAnPTzx2LnFg84tNpWp4TZBFGQz\n\
++8yTnc4kmz75fS/jY2MMddj2gbICrsRhetPfHtXV/WVhJDP1H18GbtCFY2VVPe0a\n\
+87VXE15/V8k1mE8McODmi3fipona8+/och3xWKE2rec1MKzKT0g6eXq8CrGCsyT7\n\
+YdEIqUuyyOP7uWrat2DX9GgdT0Kj3jlN9K5W7edjcrsZCwenyO4KbXCeAvzhzffi\n\
+7MA0BM0oNC9hkXL+nOmFg/+OTxIy7vKBg8P+OxtMb61zO7X8vC7CIAXFjvGDfRaD\n\
+ssbzSibBsu/6iGtCOGEfz9zeNVs7ZRkDW7w09N75nAI4YbRvydbmyQd62R0mkff3\n\
+7lmMsPrBhtkcrv4TCYUTknC0EwyTvEN5RPT9RFLi103TZPLiHnH1S/9croKrnJ32\n\
+nuhtK8UiNjoNq8Uhl5sN6todv5pC1cRITgq80Gv6U93vPBsg7j/VnXwl5B0rZp4e\n\
+8W5vUsMWTfT7eTDp5OWIV7asfV9C1p9tGHdjzx1VA0AEh/VbpX4xzHpxNciG77Qx\n\
+iu1qHgEtnmgyqQdgCpGBMMRtx3j5ca0AOAkpmaMzy4t6Gh25PXFAADwqTs6p+Y0K\n\
+zAqCkc3OyX3Pjsm1Wn+IpGtNtahR9EGC4caKAH5eZV9q//////////8CAQI=\n\
 -----END DH PARAMETERS-----\n";
 
-static const char file_dh2048[] =
+static const char file_ffdhe8192[] = \
 "-----BEGIN DH PARAMETERS-----\n\
-MIIBCAKCAQEA9kJXtwh/CBdyorrWqULzBej5UxE5T7bxbrlLOCDaAadWoxTpj0BV\n\
-89AHxstDqZSt90xkhkn4DIO9ZekX1KHTUPj1WV/cdlJPPT2N286Z4VeSWc39uK50\n\
-T8X8dryDxUcwYc58yWb/Ffm7/ZFexwGq01uejaClcjrUGvC/RgBYK+X0iP1YTknb\n\
-zSC0neSRBzZrM2w4DUUdD3yIsxx8Wy2O9vPJI8BD8KVbGI2Ou1WMuF040zT9fBdX\n\
-Q6MdGGzeMyEstSr/POGxKUAYEY18hKcKctaGxAMZyAcpesqVDNmWn6vQClCbAkbT\n\
-CD1mpF1Bn5x8vYlLIhkmuquiXsNV6TILOwIBAg==\n\
------END DH PARAMETERS-----\n";
-
-static const char file_dh4096[] =
-"-----BEGIN DH PARAMETERS-----\n\
-MIICCAKCAgEA+hRyUsFN4VpJ1O8JLcCo/VWr19k3BCgJ4uk+d+KhehjdRqNDNyOQ\n\
-l/MOyQNQfWXPeGKmOmIig6Ev/nm6Nf9Z2B1h3R4hExf+zTiHnvVPeRBhjdQi81rt\n\
-Xeoh6TNrSBIKIHfUJWBh3va0TxxjQIs6IZOLeVNRLMqzeylWqMf49HsIXqbcokUS\n\
-Vt1BkvLdW48j8PPv5DsKRN3tloTxqDJGo9tKvj1Fuk74A+Xda1kNhB7KFlqMyN98\n\
-VETEJ6c7KpfOo30mnK30wqw3S8OtaIR/maYX72tGOno2ehFDkq3pnPtEbD2CScxc\n\
-alJC+EL7RPk5c/tgeTvCngvc1KZn92Y//EI7G9tPZtylj2b56sHtMftIoYJ9+ODM\n\
-sccD5Piz/rejE3Ome8EOOceUSCYAhXn8b3qvxVI1ddd1pED6FHRhFvLrZxFvBEM9\n\
-ERRMp5QqOaHJkM+Dxv8Cj6MqrCbfC4u+ZErxodzuusgDgvZiLF22uxMZbobFWyte\n\
-OvOzKGtwcTqO/1wV5gKkzu1ZVswVUQd5Gg8lJicwqRWyyNRczDDoG9jVDxmogKTH\n\
-AaqLulO7R8Ifa1SwF2DteSGVtgWEN8gDpN3RBmmPTDngyF2DHb5qmpnznwtFKdTL\n\
-KWbuHn491xNO25CQWMtem80uKw+pTnisBRF/454n1Jnhub144YRBoN8CAQI=\n\
+MIIECAKCBAEA//////////+t+FRYortKmq/cViAnPTzx2LnFg84tNpWp4TZBFGQz\n\
++8yTnc4kmz75fS/jY2MMddj2gbICrsRhetPfHtXV/WVhJDP1H18GbtCFY2VVPe0a\n\
+87VXE15/V8k1mE8McODmi3fipona8+/och3xWKE2rec1MKzKT0g6eXq8CrGCsyT7\n\
+YdEIqUuyyOP7uWrat2DX9GgdT0Kj3jlN9K5W7edjcrsZCwenyO4KbXCeAvzhzffi\n\
+7MA0BM0oNC9hkXL+nOmFg/+OTxIy7vKBg8P+OxtMb61zO7X8vC7CIAXFjvGDfRaD\n\
+ssbzSibBsu/6iGtCOGEfz9zeNVs7ZRkDW7w09N75nAI4YbRvydbmyQd62R0mkff3\n\
+7lmMsPrBhtkcrv4TCYUTknC0EwyTvEN5RPT9RFLi103TZPLiHnH1S/9croKrnJ32\n\
+nuhtK8UiNjoNq8Uhl5sN6todv5pC1cRITgq80Gv6U93vPBsg7j/VnXwl5B0rZp4e\n\
+8W5vUsMWTfT7eTDp5OWIV7asfV9C1p9tGHdjzx1VA0AEh/VbpX4xzHpxNciG77Qx\n\
+iu1qHgEtnmgyqQdgCpGBMMRtx3j5ca0AOAkpmaMzy4t6Gh25PXFAADwqTs6p+Y0K\n\
+zAqCkc3OyX3Pjsm1Wn+IpGtNtahR9EGC4caKAH5eDdkCC/1ktkUDbHpOZ30sOFMq\n\
+OiO6RELK9T6mO7RUMpt2JMiRe91kscD9TLOOjDNMcBw6za0GV/zP7HGbH1w+TkYE\n\
+HziBR/tM/bR3pSRx96mpaRC4VTIu22NA2KAO8JI1BRHjCr7B//njom5/sp+MGDAj\n\
+w1h+ONoAd9m0dj5OS5Syu8GUxmUed8r5ku6qwCMqKBv2s6c5wSJhFoIK6NtYR6Z8\n\
+vvnJCRtGLVOM1ysDdGrnf15iKSwxFWKoRlBdyC24VDOK5J9SNclbkReMzy3Vys70\n\
+A+ydGBDGJysEWztx+dxrgNY/3UqOmtseaWKmlSbUMWHBpB1XDXk42tSkDjKcz/Rq\n\
+qjatAEz2AMg4HkJaMdlRrmT9sj/OyVCdQ2h/62nt0cxeC4zDvfZLEO+GtjFCo6uI\n\
+KVVbL3R8kyZlyywPHMAb1wIpOIg50q8F5FRQSseLdYKCKEbAujXDX1xZFgzARv2C\n\
+UVQfxoychrAiu3CZh2pGDnRRqKkxCXA/7hwhfmw4JuUsUappHg5CPPyZ6eMWUMEh\n\
+e2JIFs2tmpX51bgBlIjZwKCh/jB1pXfiMYP4HUo/L6RXHvyM4LqKT+i2hV3+crCm\n\
+bt7S+6v75Yow+vq+HF1xqH4vdB74wf6G/qa7/eUwZ38Nl9EdSfeoRD0IIuUGqfRh\n\
+TgEeKpSDj/iM1oyLt8XGQkz//////////wIBAg==\n\
 -----END DH PARAMETERS-----\n";
 
 
@@ -167,9 +184,12 @@ load_dh_buffer (const char *buffer, size_t len)
 
 
 /*
-**  Generate empheral DH key.  Because this can take a long
-**  time to compute, we use precomputed parameters of the
-**  common key sizes.
+**  Generate ephemeral DH key.  Because this can take a long
+**  time to compute, we use precomputed parameters of the common
+**  key sizes.
+**  Depending on OpenSSL Security Level, a minimal length for
+**  DH parameters is required:
+**  <https://www.openssl.org/docs/man1.1.1/man3/SSL_CTX_set_security_level.html>
 **
 **  These values can be static (once loaded or computed) since
 **  the OpenSSL library can effectively generate random keys
@@ -177,54 +197,56 @@ load_dh_buffer (const char *buffer, size_t len)
 **
 **  EDH keying is slightly less efficient than static RSA keying,
 **  but it offers Perfect Forward Secrecy (PFS).
-**
-**  FIXME:  support user-specified files, to eliminate risk of
-**  "small group" attacks.
 */
 static DH *
-tmp_dh_cb(SSL *s UNUSED, int export UNUSED, int keylength)
+tmp_dh_cb(SSL *s UNUSED, int export UNUSED, int keylength UNUSED)
 {
-	DH *r = NULL;
-	static DH *dh = NULL;
-	static DH *dh512 = NULL;
-	static DH *dh1024 = NULL;
-	static DH *dh2048 = NULL;
-	static DH *dh4096 = NULL;
+    DH *r = NULL;
 
-	switch (keylength)
-	{
-	case 512:
-		if (dh512 == NULL)
-			dh512 = load_dh_buffer(file_dh512, sizeof file_dh512);
-		r = dh512;
-		break;
-	case 1024:
-		if (dh1024 == NULL)
-			dh1024 = load_dh_buffer(file_dh1024, sizeof file_dh1024);
-		r = dh1024;
-		break;
-	case 2048:
-		if (dh2048 == NULL)
-			dh2048 = load_dh_buffer(file_dh2048, sizeof file_dh2048);
-		r = dh2048;
-		break;
-	case 4096:
-		if (dh4096 == NULL)
-			dh4096 = load_dh_buffer(file_dh4096, sizeof file_dh4096);
-		r = dh4096;
-		break;
-	default:
-		/* We should check current keylength vs. requested keylength
-		 * also, this is an extremely expensive operation! */
-                dh = DH_new();
-                if (dh != NULL) {
-                    DH_generate_parameters_ex(dh, keylength, DH_GENERATOR_2, NULL);
-                }
-		r = dh;
-	}
+    static DH *ffdhe2048 = NULL;
+    static DH *ffdhe4096 = NULL;
+    static DH *ffdhe8192 = NULL;
+    int level = 2; /* Default security level. */
 
-	return r;
+    /* Security levels have been introduced in OpenSSL 1.1.0.
+     * Well, as this part of code is no longer active for that version,
+     * only keep it for possible future re-use. */
+#if OPENSSL_VERSION_NUMBER >= 0x010100000L && !defined(LIBRESSL_VERSION_NUMBER)
+    level = SSL_get_security_level(s);
+#endif
+
+    switch(level)
+    {
+        case 0: /* Everything is permitted. */
+        case 1: /* DH keys shorter than 1024 bits are prohibited. */
+        case 2: /* DH keys shorter than 2048 bits are prohibited. */
+            if (ffdhe2048 == NULL) {
+                ffdhe2048 = load_dh_buffer(file_ffdhe2048,
+                                           sizeof(file_ffdhe2048));
+            }
+            r = ffdhe2048;
+            break;
+
+        case 3: /* DH keys shorter than 3072 bits are prohibited. */
+            if (ffdhe4096 == NULL) {
+                ffdhe4096 = load_dh_buffer(file_ffdhe4096,
+                                           sizeof(file_ffdhe4096));
+            }
+            r = ffdhe4096;
+            break;
+
+        case 4: /* DH keys shorter than 7680 bits are prohibited. */
+        default:
+            if (ffdhe8192 == NULL) {
+                ffdhe8192 = load_dh_buffer(file_ffdhe8192,
+                                           sizeof(file_ffdhe8192));
+            }
+            r = ffdhe8192;
+    }
+
+    return r;
 }
+#endif /* OpenSSL < 1.1.0 */
 
 
 /*
@@ -378,9 +400,8 @@ set_cert_stuff(SSL_CTX * ctx, char *cert_file, char *key_file)
     struct stat buf;
 
     if (cert_file != NULL) {
-	if (SSL_CTX_use_certificate_file(ctx, cert_file,
-					 SSL_FILETYPE_PEM) <= 0) {
-	    syslog(L_ERROR, "unable to get certificate from '%s'", cert_file);
+	if (SSL_CTX_use_certificate_chain_file(ctx, cert_file) <= 0) {
+	    syslog(L_ERROR, "unable to get certificates from '%s'", cert_file);
 	    return (0);
 	}
 	if (key_file == NULL)
@@ -419,10 +440,13 @@ set_cert_stuff(SSL_CTX * ctx, char *cert_file, char *key_file)
 }
 
 
-#ifdef HAVE_OPENSSL_ECC
+#if defined(HAVE_OPENSSL_ECC) && OPENSSL_VERSION_NUMBER < 0x01010100fL
 /*
 **  Provide an ECKEY from a curve name.
 **  Accepts a NULL pointer as the name.
+**  The EC_KEY_new_by_curve_name() function has been deprecated in
+**  OpenSSL 3.0.0; another mechanism to select groups has been available
+**  since OpenSSL 1.1.1.
 **
 **  Returns the key, or NULL on error.
 */
@@ -479,7 +503,8 @@ tls_init_serverengine(int verifydepth, int askcert, int requirecert,
                       char *tls_CAfile, char *tls_CApath, char *tls_cert_file,
                       char *tls_key_file, bool prefer_server_ciphers,
                       bool tls_compression, struct vector *tls_proto_vect,
-                      char *tls_ciphers, char *tls_ec_curve UNUSED)
+                      char *tls_ciphers, char *tls_ciphers13 UNUSED,
+                      char *tls_ec_curve UNUSED)
 {
     int     off = 0;
     int     verify_flags = SSL_VERIFY_NONE;
@@ -490,9 +515,6 @@ tls_init_serverengine(int verifydepth, int askcert, int requirecert,
     struct stat buf;
     size_t  tls_protos = 0;
     size_t  i;
-#ifdef HAVE_OPENSSL_ECC
-    EC_KEY *eckey;
-#endif
 
     if (tls_serverengine)
       return (0);				/* Already running. */
@@ -556,7 +578,13 @@ tls_init_serverengine(int verifydepth, int askcert, int requirecert,
     if (stat("/dev/urandom", &buf) == 0)
       RAND_load_file("/dev/urandom", 16 * 1024);
 
+/* Safe prime groups were introduced in OpenSSL 1.1.0. */
+#if OPENSSL_VERSION_NUMBER < 0x010100000L
     SSL_CTX_set_tmp_dh_callback(CTX, tmp_dh_cb);
+#else
+    SSL_CTX_set_dh_auto(CTX, 1);
+#endif
+
     SSL_CTX_set_options(CTX, SSL_OP_SINGLE_DH_USE);
 
 #ifdef HAVE_OPENSSL_ECC
@@ -565,16 +593,31 @@ tls_init_serverengine(int verifydepth, int askcert, int requirecert,
     /* We set a curve here by name if provided
      * or we use OpenSSL (>= 1.0.2) auto-selection
      * or we default to NIST P-256. */
-    eckey = eckey_from_name(tls_ec_curve);
-    if (eckey != NULL) {
-        SSL_CTX_set_tmp_ecdh(CTX, eckey);
-    } else {
-# ifdef SSL_CTX_set_ecdh_auto
-        SSL_CTX_set_ecdh_auto(CTX, 1);
+    if (tls_ec_curve != NULL) {
+# if OPENSSL_VERSION_NUMBER < 0x01010100fL
+        /* A new mechanism to select groups has been introduced
+         * in OpenSSL 1.1.1. */
+        EC_KEY *eckey;
+        eckey = eckey_from_name(tls_ec_curve);
+        if (eckey != NULL) {
+            SSL_CTX_set_tmp_ecdh(CTX, eckey);
+        }
 # else
+        if (!SSL_CTX_set1_groups_list(CTX, tls_ec_curve))
+            syslog(L_ERROR, "tlseccurve '%s' not found", tls_ec_curve);
+# endif
+    } else {
+# if OPENSSL_VERSION_NUMBER < 0x010100000L
+#  if OPENSSL_VERSION_NUMBER >= 0x01000200fL
+        /* Function supported since OpenSSL 1.0.2.
+         * Removed since OpenSSL 1.1.0, supporting ECDH by default with
+         * the most appropriate parameters. */
+        SSL_CTX_set_ecdh_auto(CTX, 1);
+#  else
         SSL_CTX_set_tmp_ecdh(CTX,
                              EC_KEY_new_by_curve_name(NID_X9_62_prime256v1));
-# endif /* SSL_CTX_set_ecdh_auto */
+#  endif /* SSL_CTX_set_ecdh_auto */
+# endif /* OpenSSL version < 1.1.0 */
      }
 #endif /* HAVE_OPENSSL_ECC */
 
@@ -611,21 +654,26 @@ tls_init_serverengine(int verifydepth, int askcert, int requirecert,
             }
         }
     } else {
-        /* Default value:  allow only TLS protocols. */
-        tls_protos = (INN_TLS_TLSv1 | INN_TLS_TLSv1_1 | INN_TLS_TLSv1_2
-                      | INN_TLS_TLSv1_3);
+        /* Default value:  allow only secure TLS protocols. */
+        tls_protos = (INN_TLS_TLSv1_2 | INN_TLS_TLSv1_3);
     }
 
     if ((tls_protos & INN_TLS_SSLv2) == 0) {
+#ifdef SSL_OP_NO_SSLv2
         SSL_CTX_set_options(CTX, SSL_OP_NO_SSLv2);
+#endif
     }
 
     if ((tls_protos & INN_TLS_SSLv3) == 0) {
+#ifdef SSL_OP_NO_SSLv3
         SSL_CTX_set_options(CTX, SSL_OP_NO_SSLv3);
+#endif
     }
 
     if ((tls_protos & INN_TLS_TLSv1) == 0) {
+#ifdef SSL_OP_NO_TLSv1
         SSL_CTX_set_options(CTX, SSL_OP_NO_TLSv1);
+#endif
     }
 
     if ((tls_protos & INN_TLS_TLSv1_1) == 0) {
@@ -653,6 +701,19 @@ tls_init_serverengine(int verifydepth, int askcert, int requirecert,
         }
     }
 
+#if OPENSSL_VERSION_NUMBER >= 0x01010100fL && defined(SSL_OP_NO_TLSv1_3)
+    /* New API added in OpenSSL 1.1.1 for TLSv1.3 cipher suites.
+     * Also check that SSL_OP_NO_TLSv1_3 is defined (to fix a build
+     * issue with LibreSSL considered as version 2, but without the
+     * same API as OpenSSL). */
+    if (tls_ciphers13 != NULL) {
+        if (SSL_CTX_set_ciphersuites(CTX, tls_ciphers13) == 0) {
+            syslog(L_ERROR, "TLS engine: cannot set ciphersuites");
+            return (-1);
+        }
+    }
+#endif
+
     if (tls_compression) {
 #if defined(SSL_OP_NO_COMPRESSION) && OPENSSL_VERSION_NUMBER >= 0x0009080dfL
         /* Function first added in OpenSSL 0.9.8m. */
@@ -709,6 +770,7 @@ tls_init(void)
                                        innconf->tlscompression,
                                        innconf->tlsprotocols,
                                        innconf->tlsciphers,
+                                       innconf->tlsciphers13,
                                        innconf->tlseccurve);
 
     if (ssl_result == -1) {
@@ -729,8 +791,13 @@ tls_init(void)
 
 
 /*
-**  Taken from OpenSSL apps/s_cb.c.
+**  Taken from OpenSSL apps/lib/s_cb.c.
+**
+**  The prototype of the callback function changed with BIO_set_callback_ex()
+**  introduced in OpenSSL 1.1.1
 */
+#if OPENSSL_VERSION_NUMBER < 0x01010100fL \
+    || defined(LIBRESSL_VERSION_NUMBER)
 static long
 bio_dump_cb(BIO * bio, int cmd, const char *argp, int argi, long argl UNUSED,
             long ret)
@@ -752,7 +819,42 @@ bio_dump_cb(BIO * bio, int cmd, const char *argp, int argi, long argl UNUSED,
     }
     return (ret);
 }
+#else
+static long
+bio_dump_cb(BIO * bio, int cmd, const char *argp, size_t len, int argi UNUSED,
+            long argl UNUSED, int ret, size_t *processed)
+{
+    if (!do_dump)
+	return (ret);
 
+    if (cmd == (BIO_CB_READ | BIO_CB_RETURN)) {
+        if (ret > 0 && processed != NULL) {
+            syslog(L_NOTICE, "read from %08lX [%08lX] (%lu bytes => %lu (0x%lX))",
+                   (unsigned long) bio, (unsigned long) argp,
+                   (unsigned long) len, (unsigned long) *processed,
+                   (unsigned long) *processed);
+            tls_dump(argp, (int) *processed);
+        } else {
+            syslog(L_NOTICE, "read from %08lX [%08lX] (%lu bytes => %d (0x%lX))",
+                   (unsigned long) bio, (unsigned long) argp,
+                   (unsigned long) len, ret, (unsigned long) ret);
+        }
+    } else if (cmd == (BIO_CB_WRITE | BIO_CB_RETURN)) {
+        if (ret > 0 && processed != NULL) {
+            syslog(L_NOTICE, "write to %08lX [%08lX] (%lu bytes => %lu (0x%lX))",
+                   (unsigned long) bio, (unsigned long) argp,
+                   (unsigned long) len, (unsigned long) *processed,
+                   (unsigned long) *processed);
+            tls_dump(argp, (int) *processed);
+        } else {
+            syslog(L_NOTICE, "write to %08lX [%08lX] (%lu bytes => %d (0x%lX))",
+                   (unsigned long) bio, (unsigned long) argp,
+                   (unsigned long) len, ret, (unsigned long) ret);
+        }
+    }
+    return (ret);
+}
+#endif
 
 /*
 **  This is the actual startup routine for the connection.  We expect
@@ -817,8 +919,16 @@ tls_start_servertls(int readfd, int writefd)
     /* We do have an SSL_set_fd() and now suddenly a BIO_ routine is called?
      * Well there is a BIO below the SSL routines that is automatically
      * created for us, so we can use it for debugging purposes. */
-    if (tls_loglevel >= 3)
-	BIO_set_callback(SSL_get_rbio(tls_conn), bio_dump_cb);
+    if (tls_loglevel >= 3) {
+        /* BIO_set_callback() was deprecated in OpenSSL 3.0.0.
+         * BIO_set_callback_ex() was introduced in OpenSSL 1.1.1. */
+#if OPENSSL_VERSION_NUMBER < 0x01010100fL \
+    || defined(LIBRESSL_VERSION_NUMBER)
+        BIO_set_callback(SSL_get_rbio(tls_conn), bio_dump_cb);
+#else
+        BIO_set_callback_ex(SSL_get_rbio(tls_conn), bio_dump_cb);
+#endif
+    }
 
     /* Dump the negotiation for loglevels 3 and 4. */
     if (tls_loglevel >= 3)
@@ -868,8 +978,9 @@ SSL_writev (SSL *ssl, const struct iovec *vector, int count)
     bytes += vector[i].iov_len;
   /* Allocate a buffer to hold the data. */
   if (NULL == buffer) {
-    buffer = (char *) xmalloc(bytes);
-    allocsize = bytes;
+    size_t to_alloc = (bytes > 0 ? bytes : 1);
+    buffer = (char *) xmalloc(to_alloc);
+    allocsize = to_alloc;
   } else if (bytes > allocsize) {
     buffer = (char *) xrealloc (buffer, bytes);
     allocsize = bytes;
