@@ -1,15 +1,10 @@
 ##########################################################################
-# INN module for innreport (3.*).
-#
-# $Id: innreport_inn.pm 10259 2018-03-08 16:19:17Z iulius $
-#
-# Sample file tested with INN 2.5, 2.4, 2.3, 2.2, 1.7.2 and 1.5.1.
+# INN module for innreport.
 #
 # (c) 1997-2001 by Fabien Tassin <fta@sofaraway.org>.
-# See innreport for more information.
-#
-# Version 3.1.0.
+# Enhanced by several contributors since then.
 #
+# See innreport documentation for more information.
 ##########################################################################
 
 # TODO: add the map file.
@@ -473,6 +468,8 @@ sub collect($$$$$$) {
     return 1 if $left =~ /^reloading pyfilter$/o;
     return 1 if $left =~ /^reloaded pyfilter OK$/o;
     return 1 if $left =~ /^python interpreter initialized OK$/o;
+    return 1 if $left =~ /^python is not initialized$/o;
+    return 1 if $left =~ /^pyfilter .+ not installed$/o;
     return 1 if $left =~ /^python method \w+ not found$/o; 
     return 1 if $left =~ /^python: First load, so I can do initialization stuff\.$/o;
     return 1 if $left =~ /^python: filter_before_reload executing\.\.\.$/o;
@@ -873,6 +870,8 @@ sub collect($$$$$$) {
     return 1 if $left =~ /^filter: Reloading bad files/o;
     return 1 if $left =~ /^filter: Saved EMP database/o;
     return 1 if $left =~ /^filter: Restored EMP database/o;
+    # PyClean status reports
+    return 1 if $left =~ /^python: pyclean successfully hooked into INN/o;
   }
   ########
   ## innfeed
@@ -1163,19 +1162,19 @@ sub collect($$$$$$) {
       my $server = $1;
       $server = lc $server unless $CASE_SENSITIVE;
       $innxmit_ihfail{$server} = 1;
-      if ($left = /436 \S+ NNTP \S+ out of space/o) {
+      if ($left =~ /436 \S+ NNTP \S+ out of space/o) {
 	$innxmit_nospace{$server}++;
 	return 1;
       }
-      if ($left = /400 \S+ space/o) {
+      if ($left =~ /400 \S+ space/o) {
 	$innxmit_nospace{$server}++;
 	return 1;
       }
-      if ($left = /400 Bad file/o) {
+      if ($left =~ /400 Bad file/o) {
 	$innxmit_crefused{$server}++;
 	return 1;
       }
-      if ($left = /480 Transfer permission denied/o) {
+      if ($left =~ /480 Transfer permission denied/o) {
 	$innxmit_crefused{$server}++;
 	return 1;
       }
@@ -1662,6 +1661,15 @@ sub collect($$$$$$) {
       $nnrpd_timeout{$cust}++;
       return 1;
     }
+    # can't read (without any other argument, it's a timeout)
+    if ($left =~ /(\S+) can\'t read$/o) {
+      my $cust = $1;
+      $cust = lc $cust unless $CASE_SENSITIVE;
+      my $dom = &host2dom($cust);
+      $nnrpd_dom_timeout{$dom}++;
+      $nnrpd_timeout{$cust}++;
+      return 1;
+    }
     # can't read: Connection timed out
     if ($left =~ /(\S+) can\'t read: Connection timed out$/o) {
       my $cust = $1;
@@ -1705,12 +1713,13 @@ sub collect($$$$$$) {
     }
     # can't getpeername
     if ($left =~ /\? can\'t getpeername/o) {
-      # $nnrpd_getpeername++;
       $nnrpd_gethostbyaddr{"? (can't getpeername)"}++;
       return 1;
     }
     # can't getsockname
     return 1 if $left =~ /^\S+ can\'t getsockname$/o;
+    # can't initialize TLS session
+    return 1 if $left =~ /^\S+ failure to negotiate TLS session$/o;
     # reverse lookup failed
     return 1 if $left =~ /^\? reverse lookup for \S+ failed: .* -- using IP address for access$/o;
     # profile timer
@@ -1746,10 +1755,9 @@ sub collect($$$$$$) {
     return 1 if $left =~ /^\S+ newnews /o;
     # cant fopen (ignored too)
     return 1 if $left =~ /^\S+ cant fopen /o;
-    # can't read: No route to host
-    return 1 if $left =~ /can\'t read: No route to host/o;
-    # can't read: Broken pipe
-    return 1 if $left =~ /can\'t read: Broken pipe/o;
+    # can't read: ... (skip any English or localized strings not already
+    # dealt with above)
+    return 1 if $left =~ /can\'t read: /o;
     # eof in post
     return 1 if $left =~ /^\S+ EOF in post$/o;
     # ioctl: ...
@@ -1876,22 +1884,12 @@ sub collect($$$$$$) {
       $rnews_host{$host}++;
       return 1;
     }
-    # rejected 437 ECP rejected
-    return 1 if $left =~ m/rejected (?:437|439) ECP rejected/o;
     # rejected 437 "Subject" header too long
     return 1 if $left =~ m/header too long/o;
-    # rejected 437 Too long line in header 1163 bytes
-    return 1 if $left =~ m/rejected (?:437|439) Too long line in header/o;
-    # rejected 437 Too many newsgroups (meow)
-    return 1 if $left =~ m/rejected (?:437|439) Too many newsgroups/o;
-    # rejected 437 Space before colon in "<a" header
-    return 1 if $left =~ m/rejected (?:437|439) Space before colon in/o;
-    # rejected 437 EMP (phl)
-    return 1 if $left =~ m/rejected (?:437|439) EMP/o;
-    # rejected 437 Scoring filter (8)
-    return 1 if $left =~ m/rejected (?:437|439) Scoring filter/o;
-    # bad_article missing Message-ID
-    return 1 if $left =~ m/bad_article missing Message-ID/o;
+    # rejected 437 Reason...
+    return 1 if $left =~ m/rejected (?:437|439)/o;
+    # bad_article missing (Message-ID|Path|...)
+    return 1 if $left =~ m/bad_article missing /o;
     # cant unspool saving to xxx
     return 1 if $left =~ m/cant unspool saving to/o;
   }
