@@ -1,6 +1,5 @@
-/*  $Id: cnfs.h 6530 2003-10-21 00:31:33Z rra $
-**
-**  cyclic news file system header
+/*
+**  Storage manager module header for Cyclic News File System method.
 */
 
 #ifndef __CNFS_H__
@@ -15,6 +14,7 @@ bool cnfs_cancel(TOKEN token);
 bool cnfs_ctl(PROBETYPE type, TOKEN *token, void *value);
 bool cnfs_flushcacheddata(FLUSHTYPE type);
 void cnfs_printfiles(FILE *file, TOKEN token, char **xref, int ngroups);
+char *cnfs_explaintoken(const TOKEN token);
 void cnfs_shutdown(void);
 
 #endif
