@@ -1,5 +1,4 @@
-/*  $Id: ov.c 8300 2009-01-29 18:48:53Z iulius $
-**
+/*
 **  The implementation of the overview API.
 **
 **  This code handles calls to the overview API by passing them along to the
@@ -28,15 +27,14 @@
 static bool             OVdelayrm;
 static OV_METHOD	ov;
 
-time_t	OVrealnow;
-bool    OVstatall;
+time_t	OVrealnow = 0;
+bool    OVstatall = false;
 
 bool
 OVopen(int mode)
 {
     int	i;
     bool val;
-    char *p;
 
     if (ov.open)
 	/* already opened */
@@ -68,17 +66,6 @@ OVopen(int mode)
 	OVclose();
 	return false;
     }
-    if (innconf->ovgrouppat != NULL) {
-	for (i = 1, p = innconf->ovgrouppat; *p && (p = strchr(p+1, ',')); i++);
-	OVnumpatterns = i;
-	OVpatterns = xmalloc(OVnumpatterns * sizeof(char *));
-	for (i = 0, p = strtok(innconf->ovgrouppat, ","); p != NULL && i <= OVnumpatterns ; i++, p = strtok(NULL, ","))
-	    OVpatterns[i] = xstrdup(p);
-	if (i != OVnumpatterns) {
-            warn("extra ',' in pattern");
-	    return false;
-	}
-    }
     return val;
 }
 
@@ -129,22 +116,24 @@ OVadd(TOKEN token, char *data, int len, time_t arrived, time_t expires)
     int			i;
     char		*group;
     ARTNUM		artnum;
+    enum uwildmat       groupmatch;
 
     if (!ov.open) {
-	/* must be opened */
+	/* Must be opened. */
         warn("ovopen must be called first");
 	return OVADDFAILED;
     }
 
     /*
-     * find last Xref: in the overview line.  Note we need to find the *last*
+     * Find last Xref: in the overview line.  Note we need to find the *last*
      * Xref:, since there have been corrupted articles on Usenet with Xref:
      * fragments stuck in other header lines.  The last Xref: is guaranteed
      * to be from our server.
      */
 
     for (next = data; ((len - (next - data)) > 6 ) && ((next = memchr(next, 'X', len - (next - data))) != NULL); ) {
-        if (memcmp(next, "Xref: ", 6) == 0) {
+        /* Check that Xref: is at the beginning of an overview field. */
+        if (memcmp(next, "Xref: ", 6) == 0 && next != data && next[-1] == '\t') {
             found =  true;
             xrefstart = next;
         }
@@ -163,8 +152,8 @@ OVadd(TOKEN token, char *data, int len, time_t arrived, time_t expires)
     xreflen = len - (next - data);
 
     /*
-     * If there are other fields beyond Xref in overview, then
-     * we must find Xref's end, or data following is misinterpreted.
+     * If there are other fields beyond Xref: in overview, then
+     * we must find Xref:'s end, or data following is misinterpreted.
      */
     if ((xrefend = memchr(next, '\t', xreflen)) != NULL)
 	xreflen = xrefend - next;
@@ -194,26 +183,30 @@ OVadd(TOKEN token, char *data, int len, time_t arrived, time_t expires)
         memcpy(patcheck, next, xreflen);
         patcheck[xreflen] = '\0';
         for (group = patcheck; group && *group; group = memchr(nextcheck, ' ', xreflen - (nextcheck - patcheck))) {
-            while (isspace((int)*group))
+            while (isspace((unsigned char) *group))
                 group++;
-            if ((nextcheck = memchr(group, ':', xreflen - (patcheck - group))) == NULL)
+            if ((nextcheck = memchr(group, ':', xreflen - (group - patcheck))) == NULL)
                 return OVADDFAILED;
             *nextcheck++ = '\0';
-            if (!OVgroupmatch(group)) {
-                if (!SMprobe(SELFEXPIRE, &token, NULL) && innconf->groupbaseexpiry)
-                    /* this article will never be expired, since it does not
-                       have self expiry function in stored method and
-                       groupbaseexpiry is true */
-                    return OVADDFAILED;
+
+            groupmatch = uwildmat_poison(group, innconf->ovgrouppat);
+            if (groupmatch == UWILDMAT_POISON) {
                 return OVADDGROUPNOMATCH;
+            } else if (groupmatch == UWILDMAT_FAIL) {
+                if (!SMprobe(SELFEXPIRE, &token, NULL) && innconf->groupbaseexpiry) {
+                    /* This article will never be expired, since it does not
+                     * have self expiry function in stored method and
+                     * groupbaseexpiry is true. */
+                    return OVADDFAILED;
+                }
             }
         }
     }
     memcpy(xrefdata, next, xreflen);
     xrefdata[xreflen] = '\0';
     for (group = xrefdata; group && *group; group = memchr(next, ' ', xreflen - (next - xrefdata))) {
-        /* Parse the xref part into group name and article number */
-        while (isspace((int)*group))
+        /* Parse the Xref: part into group name and article number. */
+        while (isspace((unsigned char) *group))
             group++;
         if ((next = memchr(group, ':', xreflen - (group - xrefdata))) == NULL)
             return OVADDFAILED;
@@ -222,6 +215,12 @@ OVadd(TOKEN token, char *data, int len, time_t arrived, time_t expires)
         if (artnum <= 0)
             continue;
 
+        /* Skip overview generation according to ovgrouppat. */
+        if (innconf->ovgrouppat != NULL
+            && uwildmat_poison(group, innconf->ovgrouppat) != UWILDMAT_MATCH) {
+            continue;
+        }
+
         sprintf(overdata, "%ld\t", artnum);
         i = strlen(overdata);
         memcpy(overdata + i, data, len);
@@ -242,7 +241,7 @@ OVcancel(TOKEN token)
     ARTHANDLE *art;
     const char *xref, *xrefend, *group;
     size_t xreflen, i;
-    char *xref_copy, *p, *end;
+    char *xref_copy, *p;
     ARTNUM artnum;
     struct cvector *groups;
 
@@ -259,7 +258,7 @@ OVcancel(TOKEN token)
     art = SMretrieve(token, RETR_HEAD);
     if (art == NULL)
         return false;
-    xref = wire_findheader(art->data, art->len, "Xref");
+    xref = wire_findheader(art->data, art->len, "Xref", true);
     if (xref == NULL)
         goto fail;
     xrefend = wire_endheader(xref, art->data + art->len - 1);
@@ -276,8 +275,8 @@ OVcancel(TOKEN token)
             continue;
         *p = '\0';
         errno = 0;
-        artnum = strtoul(p + 1, &end, 10);
-        if (artnum == 0 || *end != '\0' || errno == ERANGE)
+        artnum = strtoul(p + 1, NULL, 10);
+        if (artnum == 0 || errno == ERANGE)
             continue;
 
         /* Don't worry about the return status; the article may have already
