@@ -1,6 +1,5 @@
-/*  $Id: timehash.h 6530 2003-10-21 00:31:33Z rra $
-**
-**  timehash based storing method header
+/*
+**  Storage manager module header for timehash method.
 */
 
 #ifndef __TIMEHASH_H__
@@ -18,6 +17,7 @@ bool timehash_cancel(TOKEN token);
 bool timehash_ctl(PROBETYPE type, TOKEN *token, void *value);
 bool timehash_flushcacheddata(FLUSHTYPE type);
 void timehash_printfiles(FILE *file, TOKEN token, char **xref, int ngroups);
+char *timehash_explaintoken(const TOKEN token);
 void timehash_shutdown(void);
 
 #endif
