@@ -1,5 +1,4 @@
-/*  $Id: tdx-data.c 8481 2009-05-20 20:10:04Z iulius $
-**
+/*
 **  Overview data file handling for the tradindexed overview method.
 **
 **  Implements the handling of the .IDX and .DAT files for the tradindexed
@@ -22,6 +21,7 @@
 #include <fcntl.h>
 #include <sys/stat.h>
 
+#include "inn/fdflag.h"
 #include "inn/history.h"
 #include "inn/innconf.h"
 #include "inn/messages.h"
@@ -179,7 +179,7 @@ file_open_index(struct group_data *data, const char *suffix)
         return false;
     }
     data->indexinode = st.st_ino;
-    close_on_exec(data->indexfd, true);
+    fdflag_close_exec(data->indexfd, true);
     return true;
 }
 
@@ -198,7 +198,7 @@ file_open_data(struct group_data *data, const char *suffix)
     data->datafd = file_open(data->path, suffix, data->writable, true);
     if (data->datafd < 0)
         return false;
-    close_on_exec(data->datafd, true);
+    fdflag_close_exec(data->datafd, true);
     return true;
 }
 
@@ -425,6 +425,8 @@ tdx_article_entry(struct group_data *data, ARTNUM article, ARTNUM high)
         unmap_index(data);
         map_index(data);
         data->high = high;
+        /* Mark the data file to be remapped for the next opensearch. */
+        data->remapoutoforder = true;
     } else if (innconf->nfsreader && stale_index(data))
         unmap_index(data);
     if (data->index == NULL)
@@ -462,8 +464,6 @@ tdx_search_open(struct group_data *data, ARTNUM start, ARTNUM end, ARTNUM high)
     if ((end > data->high && high > data->high) || data->remapoutoforder) {
         data->remapoutoforder = false;
         unmap_data(data);
-    }
-    if (end > data->high && high > data->high) {
         unmap_index(data);
         map_index(data);
         data->high = high;
