@@ -1,5 +1,4 @@
-/*  $Id: tdx-structure.h 7585 2006-11-21 09:37:51Z eagle $
-**
+/*
 **  Data structures for the tradindexed overview method.
 **
 **  This header defines the data structures used by the tradindexed overview
@@ -30,7 +29,7 @@
 **  stored in <pathoverview>/n/a/n/news.announce.newgroups.{IDX,DAT}.  The
 **  .DAT file contains the individual overview entries, one per line, stored
 **  in wire format (in other words, suitable for dumping directly across the
-**  network to a client in response to an XOVER command).  The overview data
+**  network to a client in response to an OVER command).  The overview data
 **  stored in that file may be out of order.
 **
 **  The .IDX file consists of a series of struct index_entry's, one for each
@@ -69,7 +68,8 @@ struct loc {
 #define TDX_HASH_SIZE   (16 * 1024)
 
 /* A magic number for the group.index file so that we can later change the
-   format in a backward-compatible fashion. */
+   format in a backward-compatible fashion.
+   This magic number stands for "fifo feed". */
 #define TDX_MAGIC       (~(0xf1f0f33d))
 
 /* The header at the top of group.index.  magic contains GROUPHEADERMAGIC
