@@ -1,5 +1,4 @@
-/*  $Id: tradspool.h 8617 2009-09-07 08:21:55Z iulius $
-** 
+/*
 **  Storage manager module header for traditional spool format.
 */
 
@@ -18,6 +17,7 @@ bool tradspool_cancel(TOKEN token);
 bool tradspool_ctl(PROBETYPE type, TOKEN *token, void *value);
 bool tradspool_flushcacheddata(FLUSHTYPE type);
 void tradspool_printfiles(FILE *file, TOKEN token, char **xref, int ngroups);
+char *tradspool_explaintoken(const TOKEN token);
 void tradspool_shutdown(void);
 
 #endif
