@@ -1,15 +1,17 @@
-/* $Id: chan-t.c 7653 2007-08-22 11:51:37Z iulius $ */
 /* Test suite for general channel handling. */
 
 #include "config.h"
 #include "clibrary.h"
 #include <fcntl.h>
-#include <sys/time.h>
+#ifdef HAVE_SYS_TIME_H
+# include <sys/time.h>
+#endif
+#include <time.h>
 
 #include "inn/buffer.h"
 #include "inn/innconf.h"
 #include "inn/libinn.h"
-#include "libtest.h"
+#include "tap/basic.h"
 
 #include "../../innd/innd.h"
 
